/*
 * This file is part of SEMP.
 *
 * SEMP is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SEMP is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SEMP. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2009, 2010, 2011, 2012 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */

#include "DirectShow.h"

/**
Constructor
\param osd OSD instance to be used
\param debug enabling debugging information
*/
DirectShow::DirectShow(OSD* osd, bool debug = false)
{
	g_pGraphBuilder = NULL;
	g_pMediaControl = NULL;
	g_pMediaEvent = NULL;
	g_pMediaPosition = NULL;
	g_pBasicAudio = NULL;
	g_pMediaSeek = NULL;
	g_pWindowlessControl = NULL;
	g_pWindowlessFilter = NULL;
	g_pSourceFilter = NULL;
	g_pCaptureGraph = NULL;
	g_bDebug = debug;
	g_pOSD = osd;
	g_pSampleGrabber = NULL;
	g_pGrabberFilter = NULL;
	g_bAutoColor = false;
	g_bDisableMMX = false;
	pVih = NULL;
}

/**
Destructor
*/
DirectShow::~DirectShow()
{
	CleanUpDirectShow();
	FreeMediaType(mt);
	if (pVih != NULL)
	{
		delete pVih;
		pVih = NULL;
	}
}

/**
Getter for the VMR Windows control
\return IVMRWindowlessControl with the windowless control
*/
IVMRWindowlessControl* DirectShow::GetVMRWindowlessControl()
{
	return g_pWindowlessControl;
}

/**
Getter for the Graph builder
\return IGraphBuilder with the graph builder
*/
IGraphBuilder* DirectShow::GetGraphBuilder()
{
	return g_pGraphBuilder;
}

/**
Getter frforo the Media control
\return IMediaControl with the media control
*/
IMediaControl* DirectShow::GetMediaControl()
{
	return g_pMediaControl;
}

/**
Getter for the Media Event
\return IMediaEventEx with the media event
*/
IMediaEventEx* DirectShow::GetMediaEvent()
{
	return g_pMediaEvent;
}

/**
Getter for the Media position
\return IMediaPosition with the media position
*/
IMediaPosition* DirectShow::GetMediaPosition()
{
	return g_pMediaPosition;
}

/**
Getter for the Basic audio
\return IBasicAudio with the basic audio
*/
IBasicAudio* DirectShow::GetBasicAudio()
{
	return g_pBasicAudio;
}

/**
Getter for the Media seeking
\return IMediaSeeking with the media seeking
*/
IMediaSeeking* DirectShow::GetMediaSeek()
{
	return g_pMediaSeek;
}

/**
Getter for the Sample Grabber
\return ISampleGrabber with the sample grabber
*/
ISampleGrabber* DirectShow::GetSampleGrabber()
{
	return g_pSampleGrabber;
}

/**
Method for handling DirectShow Graph events
*/
void DirectShow::OnGraphEvent()
{
	long EventCode, Param1, Param2;
	while (g_pMediaEvent != NULL && g_pMediaEvent->GetEvent(&EventCode, &Param1, &Param2, 0) != E_ABORT)
	{
		switch (EventCode)
		{
        case EC_COMPLETE:            
            // here when media is completely done playing
            g_pMediaControl->Stop();
            g_pMediaPosition->put_CurrentPosition(0);   // reset to beginning
			PostQuitMessage(0);
            break;
		default:
			g_pMediaEvent->FreeEventParams(EventCode, Param1, Param2);
			break;
		}	
	}
}

/**
Method for initialising the capture graph builder
\param ppGraph graph builder to be created
\param ppBuild capture graph builder to be created
\return error or ok
*/
HRESULT DirectShow::InitCaptureGraphBuilder(
  IGraphBuilder **ppGraph,  // Receives the pointer.
  ICaptureGraphBuilder2 **ppBuild  // Receives the pointer.
)
{
    if (!ppGraph || !ppBuild)
    {
        return E_POINTER;
    }
    IGraphBuilder *pGraph = NULL;
    ICaptureGraphBuilder2 *pBuild = NULL;

    // Create the Capture Graph Builder.
    HRESULT hr = CoCreateInstance(CLSID_CaptureGraphBuilder2, NULL, 
        CLSCTX_INPROC_SERVER, IID_ICaptureGraphBuilder2, (void**)&pBuild );
    if (SUCCEEDED(hr))
    {
        // Create the Filter Graph Manager.
        hr = CoCreateInstance(CLSID_FilterGraph, 0, CLSCTX_INPROC_SERVER,
            IID_IGraphBuilder, (void**)&pGraph);
        if (SUCCEEDED(hr))
        {
            // Initialize the Capture Graph Builder.
            pBuild->SetFiltergraph(pGraph);

            // Return both interface pointers to the caller.
            *ppBuild = pBuild;
            *ppGraph = pGraph; // The caller must release both interfaces.
            return S_OK;
        }
        else
        {
            pBuild->Release();
        }
    }
    return hr; // Failed
}

/**
Method for initialising DirectShow
\param appWindow handle for the application
\param autoColor true if autocolor should be used
\param storeBMP true if frames should be stored as BMP
\param disableMMX true if MMX should not be used
\return -1 if error else 0
*/
int DirectShow::InitDirectShow(HWND appWindow, bool autoColor = false, bool storeBMP = false, bool disableMMX = false)
{
	g_bAutoColor = autoColor;
	g_bDisableMMX = disableMMX;
	g_bStoreBMP = storeBMP;

	HRESULT hr;

	hr = InitCaptureGraphBuilder(&g_pGraphBuilder, &g_pCaptureGraph);
    if (FAILED(hr))
        return -1;

	g_pGraphBuilder->QueryInterface(IID_IMediaControl, (void**)&g_pMediaControl);
	g_pGraphBuilder->QueryInterface(IID_IMediaEvent, (void**)&g_pMediaEvent);
	g_pGraphBuilder->QueryInterface(IID_IMediaPosition, (void**)&g_pMediaPosition);

	g_pMediaEvent->SetNotifyWindow((OAHWND)appWindow, WM_GRAPHEVENT, 0);
	g_pMediaEvent->SetNotifyFlags(0);	// turn on notifications
	
	g_pGraphBuilder->QueryInterface(IID_IBasicAudio, (void **)&g_pBasicAudio);
	g_pGraphBuilder->QueryInterface(IID_IMediaSeeking, (void **)&g_pMediaSeek);

	if (autoColor || storeBMP)
	{
		hr = CoCreateInstance(CLSID_SampleGrabber, NULL, CLSCTX_INPROC_SERVER,
			IID_IBaseFilter, (void **)(&g_pGrabberFilter));
		hr = g_pGrabberFilter->QueryInterface(IID_ISampleGrabber,
			(void **)(&g_pSampleGrabber));
		hr = g_pGraphBuilder->AddFilter(g_pGrabberFilter, L"SampleGrabber");
		if (FAILED(hr))
		{
			MessageBox(NULL, "Unable to add sample grabber!", "Error", MB_OK);
			return -1;
		}

		// Set the media type
		AM_MEDIA_TYPE mt;
		ZeroMemory(&mt, sizeof(AM_MEDIA_TYPE));
		mt.formattype = FORMAT_VideoInfo; 
		mt.majortype = MEDIATYPE_Video;

		if (!disableMMX)
			mt.subtype = MEDIASUBTYPE_RGB32; // BGR with A
		else
			mt.subtype = MEDIASUBTYPE_RGB24; // only accept 24-bit bitmaps

		hr = g_pSampleGrabber->SetMediaType(&mt);
	}

	hr = InitWindowlessVMR(appWindow, g_pGraphBuilder, &g_pWindowlessControl);
	if (FAILED(hr))
		return -1;

	long lWidth, lHeight; 
	hr = g_pWindowlessControl->GetNativeVideoSize(&lWidth, &lHeight, NULL, NULL); 
	if (SUCCEEDED(hr))
	{
		RECT rcSrc, rcDest; 
		// Set the source rectangle.
		SetRect(&rcSrc, 0, 0, lWidth, lHeight); 
	    
		// Get the window client area.
		GetClientRect(appWindow, &rcDest); 
		// Set the destination rectangle.
		SetRect(&rcDest, 0, 0, rcDest.right, rcDest.bottom); 
	    
		// Set the video position.
		hr = g_pWindowlessControl->SetVideoPosition(&rcSrc, &rcDest);

		hr = g_pWindowlessControl->SetAspectRatioMode(1); // VMRAspectRatioMode.LetterBox
	}

    return hr;
}

/**
Method for releasing DirectShow components
*/
void DirectShow::CleanUpDirectShow()
{
    HELPER_RELEASE(g_pMediaPosition);
    HELPER_RELEASE(g_pMediaEvent);
    HELPER_RELEASE(g_pMediaControl);
    HELPER_RELEASE(g_pGraphBuilder);
	HELPER_RELEASE(g_pBasicAudio);
	HELPER_RELEASE(g_pMediaSeek);
	HELPER_RELEASE(g_pWindowlessControl);
	HELPER_RELEASE(g_pSampleGrabber);
	HELPER_RELEASE(g_pGrabberFilter);
	HELPER_RELEASE(g_pWindowlessFilter);
}

/**
Method for creating a DirectShow graph
\param filename multimedia file for which the graph should be created
\param executeDir directory for storing the debugging information
\param lVolume volume for audio to start with
\param effectManager used for handling sensory effects
\return -1 if an error occured else 0
*/
int DirectShow::CreateGraph(char* filename, char* executeDir, long lVolume, EffectManager* effectManager)
{
	int	    length;		// length of filename
	WCHAR*	wfilename;	// where we store WCHAR version of filename
    HRESULT hr;
    HANDLE  logfile;                  // log file for graphbuilder
    char    logfilename[MAX_PATH];    // full path and name of log file

	if (g_bDebug == true)
	{
		// create graphbuilder's log file;  we must give it a full path or
		// else it will create the log file in the directory the media file is
		// located in
		sprintf_s(logfilename, "%s\\graph.log", executeDir);
		logfile = CreateFile(logfilename, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS,
			FILE_ATTRIBUTE_NORMAL, NULL);
		g_pGraphBuilder->SetLogFile(reinterpret_cast< DWORD_PTR >(logfile));
	}

    // attempt to build a graph
	length = strlen(filename)+1;
	wfilename = new WCHAR[length];
	MultiByteToWideChar(CP_ACP, 0, filename, -1, wfilename, length);
	
	hr = g_pGraphBuilder->AddSourceFilter(wfilename, L"Source", &g_pSourceFilter);
	if(FAILED(hr))
	{
		MessageBox(NULL, "Cannot add source filter!", "Error", MB_OK);
		delete wfilename;

		if (g_bDebug == true)
			CloseHandle(logfile);

		return -1;
	}

	// have with autocolor or do we need to store BMPs?
	// if yes we need a SampleGrabber
	if (g_bAutoColor || g_bStoreBMP)
	{
		// Connect the src and grabber
		hr = ConnectFilters(g_pGraphBuilder, g_pSourceFilter, g_pGrabberFilter);
		if(FAILED(hr))
		{
			MessageBox(NULL, "Unable to connect source to grabber filters!", "Error", MB_OK);
			delete wfilename;

			if (g_bDebug == true)
				CloseHandle(logfile);

			return -1;
		}

		// Connect the grabber with vmr
		hr = ConnectFilters(g_pGraphBuilder, g_pGrabberFilter, g_pWindowlessFilter);
		if(FAILED(hr))
		{
			MessageBox(NULL, "Unable to connect grabber to vmr filters!", "Error", MB_OK);
			delete wfilename;

			if (g_bDebug == true)
				CloseHandle(logfile);

			return -1;
		}

		// Set modes
		g_pSampleGrabber->SetBufferSamples(TRUE);
		g_pSampleGrabber->SetOneShot(FALSE);

		ZeroMemory(&mt, sizeof(mt));
		hr = g_pSampleGrabber->GetConnectedMediaType(&mt);
		if (FAILED(hr))
		{
			MessageBox(NULL, "Problem with media type", "Error", MB_OK);
			delete wfilename;
			if (g_bDebug == true)
				CloseHandle(logfile);

			return -1;
		}

		if (mt.formattype == FORMAT_VideoInfo) 
			pVih = reinterpret_cast<VIDEOINFOHEADER*>(mt.pbFormat);
		else 
		{
			MessageBox(NULL, "Problem with video information header", "Error", MB_OK);
			delete wfilename;
			if (g_bDebug == true)
				CloseHandle(logfile);

			return -1;
		}
	}
	else
	{
		// Connect the src and vmr
		hr = ConnectFilters(g_pGraphBuilder, g_pSourceFilter, g_pWindowlessFilter);
		if(FAILED(hr))
		{
			MessageBox(NULL, "Unable to connect source to vmr filters!", "Error", MB_OK);
			delete wfilename;
			
			if (g_bDebug == true)
				CloseHandle(logfile);

			return -1;
		}
	}
	
	delete wfilename;

	hr = g_pCaptureGraph->RenderStream(0, 0, g_pSourceFilter, 0, g_pWindowlessFilter);
	hr = g_pCaptureGraph->RenderStream(0, &MEDIATYPE_Audio, g_pSourceFilter, 0, NULL);

	if (g_bDebug == true)
		CloseHandle(logfile);
	

	HRESULT hr2 = g_pBasicAudio->put_Volume(lVolume);
	if (FAILED(hr2))
	{
		MessageBox(NULL, "Unable to set initial volume!", "Audio error", MB_OK);
		//OnPaint(g_AppWindow);
	}

	if (effectManager != NULL)
	{
		LONGLONG duration = 0;
		hr2 = g_pMediaSeek->GetDuration(&duration);
		if (FAILED(hr2))
		{
			MessageBox(NULL, "Unable to determine video duration!", "Video error", MB_OK);
			return -1;
		}

		effectManager->SetVideoLength(duration/HUNDRET_MSEC);
	}

	if (g_bDebug == true)
	{
		// Create graph file
		const WCHAR wszStreamName[] = L"ActiveMovieGraph";
		IStorage *pStorage = NULL;
		char graphFile[MAX_PATH];
		sprintf_s(graphFile, "%s\\graph.grf", executeDir);
		WCHAR wGraphFilename[MAX_PATH];
		MultiByteToWideChar(CP_ACP, 0, graphFile, -1, wGraphFilename, MAX_PATH);
		hr = StgCreateDocfile(wGraphFilename, STGM_CREATE | STGM_TRANSACTED | STGM_READWRITE | STGM_SHARE_EXCLUSIVE, 0, &pStorage);
		if (FAILED(hr))
			return -1;

		IStream *pStream;
		hr = pStorage->CreateStream(wszStreamName, STGM_WRITE | STGM_CREATE | STGM_SHARE_EXCLUSIVE, 0, 0, &pStream);
		if (FAILED(hr))
		{
			pStorage->Release();
			pStorage = NULL;
			return -1;
		}

		IPersistStream *pPersist = NULL;
		g_pGraphBuilder->QueryInterface(IID_IPersistStream, reinterpret_cast<void **>(&pPersist));
		hr = pPersist->Save(pStream, TRUE);
		pStream->Release();
		pPersist->Release();
		pStream = NULL;
		pPersist = NULL;
		if (SUCCEEDED(hr))
			pStorage->Commit(STGC_DEFAULT);

		pStorage->Release();
		pStorage = NULL;
	}

 	if (FAILED(hr))
		return -1;
	else
		return 0;
}

/**
Method for connecting all unconnected pins in the graph
\param pFilter filter to connect
\param PinDir which pin to connect (output or input)
\param ppPin storing the connected pin configuration
\return error or ok
*/
HRESULT DirectShow::GetUnconnectedPin(
    IBaseFilter *pFilter,   // Pointer to the filter.
    PIN_DIRECTION PinDir,   // Direction of the pin to find.
    IPin **ppPin)           // Receives a pointer to the pin.
{
    *ppPin = 0;
    IEnumPins *pEnum = 0;
    IPin *pPin = 0;
    HRESULT hr = pFilter->EnumPins(&pEnum);
    if (FAILED(hr))
    {
        return hr;
    }
    while (pEnum->Next(1, &pPin, NULL) == S_OK)
    {
        PIN_DIRECTION ThisPinDir;
        pPin->QueryDirection(&ThisPinDir);
        if (ThisPinDir == PinDir)
        {
            IPin *pTmp = 0;
            hr = pPin->ConnectedTo(&pTmp);
            if (SUCCEEDED(hr))  // Already connected, not the pin we want.
            {
                pTmp->Release();
            }
            else  // Unconnected, this is the pin we want.
            {
                pEnum->Release();
                *ppPin = pPin;
                return S_OK;
            }
        }
        pPin->Release();
    }
    pEnum->Release();
    // Did not find a matching pin.
    return E_FAIL;
}


/**
Method for connecting filters
\param pGraph the filter graph
\param pOut output to connect to
\param pDest destination filter
\return error or ok
*/
HRESULT DirectShow::ConnectFilters(
    IGraphBuilder *pGraph, // Filter Graph Manager.
    IPin *pOut,            // Output pin on the upstream filter.
    IBaseFilter *pDest)    // Downstream filter.
{
    if ((pGraph == NULL) || (pOut == NULL) || (pDest == NULL))
    {
        return E_POINTER;
    }

	if (g_bDebug)
	{
		PIN_DIRECTION PinDir;
		pOut->QueryDirection(&PinDir);
		_ASSERTE(PinDir == PINDIR_OUTPUT);
	}

    // Find an input pin on the downstream filter.
    IPin *pIn = 0;
    HRESULT hr = GetUnconnectedPin(pDest, PINDIR_INPUT, &pIn);
    if (FAILED(hr))
    {
        return hr;
    }
    // Try to connect them.
    hr = pGraph->Connect(pOut, pIn);
    pIn->Release();
    return hr;
}

/**
Method for connecting filters
\param pGraph the filter graph
\param pSrc source filter
\param pDest destination filter
\return error or ok
*/
HRESULT DirectShow::ConnectFilters(
    IGraphBuilder *pGraph, 
    IBaseFilter *pSrc, 
    IBaseFilter *pDest)
{
    if ((pGraph == NULL) || (pSrc == NULL) || (pDest == NULL))
    {
        return E_POINTER;
    }

    // Find an output pin on the first filter.
    IPin *pOut = 0;
    HRESULT hr = GetUnconnectedPin(pSrc, PINDIR_OUTPUT, &pOut);
    if (FAILED(hr)) 
    {
        return hr;
    }
    hr = ConnectFilters(pGraph, pOut, pDest);
    pOut->Release();
    return hr;
}

/**
Method for initialisating the windowless VMR
\param hwndApp window to hold the video
\param pGraph pointer to the graph manager
\param ppWc receives the pointer to the VMR
\return error or ok
*/
HRESULT DirectShow::InitWindowlessVMR( 
    HWND hwndApp,                  // Window to hold the video. 
    IGraphBuilder* pGraph,         // Pointer to the Filter Graph Manager. 
    IVMRWindowlessControl** ppWc   // Receives a pointer to the VMR.
    ) 
{ 
    if (!pGraph || !ppWc) 
    {
        return E_POINTER;
    }
    //IBaseFilter* pVmr = NULL; 
 
    // Create the VMR. 
    HRESULT hr = CoCreateInstance(CLSID_VideoMixingRenderer, NULL, 
        CLSCTX_INPROC_SERVER, IID_IBaseFilter, (void**)&g_pWindowlessFilter); 
    if (FAILED(hr))
    {
		// try another renderer
		//hr = CoCreateInstance(CLSID_VideoRendererDefault, NULL, 
		//	CLSCTX_INPROC_SERVER, IID_IBaseFilter, (void**)&g_pWindowlessFilter);
		//if (FAILED(hr))
		return hr;
    }

    // Add the VMR to the filter graph.
    hr = pGraph->AddFilter(g_pWindowlessFilter, L"Video Mixing Renderer"); 
    if (FAILED(hr)) 
    {
        g_pWindowlessFilter->Release();
        return hr;
    }
    // Set the rendering mode.  
    IVMRFilterConfig* pConfig = NULL; 
    hr = g_pWindowlessFilter->QueryInterface(IID_IVMRFilterConfig, (void**)&pConfig); 
    if (SUCCEEDED(hr)) 
    { 
        hr = pConfig->SetRenderingMode(VMRMode_Windowless);
		hr = pConfig->SetNumberOfStreams(1);
        pConfig->Release(); 
    }

    if (SUCCEEDED(hr))
    {
        // Set the window. 
		IVMRWindowlessControl* pWc = NULL; 
        hr = g_pWindowlessFilter->QueryInterface(IID_IVMRWindowlessControl, (void**)&pWc);
        if( SUCCEEDED(hr)) 
        { 
            hr = pWc->SetVideoClippingWindow(hwndApp); 
            if (SUCCEEDED(hr))
            {
				// Get alpha-blended bitmap interface
				hr = g_pOSD->SetVMRMixerBitmap(g_pWindowlessFilter);
				if (FAILED(hr))
					pWc->Release();
				else
	                *ppWc = pWc; // Return this as an AddRef'd pointer. 
            }
            else
            {
                // An error occurred, so release the interface.
                pWc->Release();
            }
        } 
    } 
    //pVmr->Release(); 
    return hr; 
} 

/**
Method for updating the audio volume
\param volume volume to set
\return error code or 0
*/
int DirectShow::UpdateVolume(long volume = 0)
{
	if (g_pBasicAudio != NULL)
		return g_pBasicAudio->put_Volume(volume);
	else
		return -1;
}

/**
Method for retrieving the AM Media Type
\return AM_MEDIA_TYPE
*/
AM_MEDIA_TYPE DirectShow::GetAMMediaType()
{
	return mt;
}

/**
Method for retrieving the video information header
\return VIDEOINFOHEADER with the information
*/
VIDEOINFOHEADER *DirectShow::GetVideoInfoHeader()
{
	return pVih;
}
