/*
 * This file is part of SEMP.
 *
 * SEMP is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SEMP is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SEMP. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2009, 2010, 2011, 2012 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */

#include "OSD.h"

// defines for the OSD
#define DEFAULT_FONT_NAME   TEXT("Impact\0")
#define DEFAULT_FONT_STYLE  TEXT("Regular\0")
#define DEFAULT_FONT_SIZE   20
#define DEFAULT_FONT_COLOR  RGB(250,250,250)

/**
Constructor
*/
OSD::OSD()
{
	g_pBMP = NULL;
	g_lFontPointSize = DEFAULT_FONT_SIZE;
	_tccpy_s(g_szFontName, NULL, DEFAULT_FONT_NAME);
	_tccpy_s(g_szFontStyle, NULL, DEFAULT_FONT_STYLE);
	g_rgbColors = DEFAULT_FONT_COLOR;
	isOSDVisible = false;
}

/**
Destructor
*/
OSD::~OSD()
{
	HELPER_RELEASE(g_pBMP);
}

/**
Method for checking of OSD is visible
\return true if visible else false
*/
bool OSD::IsOSDVisible()
{
	return isOSDVisible;
}

/**
Method for returning the VMR Mixer Bitmap
\return VMR Mixer bitmap
*/
IVMRMixerBitmap* OSD::GetVMRMixerBitmap()
{
	return g_pBMP;
}

/**
Method for setting the VMR Mixer Bitmap
\param pWindowlessFilter filter for adding the VMR Mixer Bitmap
\return error or ok
*/
HRESULT OSD::SetVMRMixerBitmap(IBaseFilter* pWindowlessFilter)
{
	return pWindowlessFilter->QueryInterface(IID_IVMRMixerBitmap, (void**)&g_pBMP);
}

/**
Method for refreshing the voting OSD
\param hwndApp application handle
\param pWindowlessControl windowless control for refreshing
\param curVote the current vote
*/
void OSD::RefreshVotingOSD(HWND hwndApp, IVMRWindowlessControl* pWindowlessControl, int curVote)
{
	BlendVotingSlider(hwndApp, pWindowlessControl, curVote);
}

/**
Method for preparing and showing the voting OSD
\param hwndApp application handle
\param pWindowlessControl windowless control for showing the OSD on
*/
void OSD::PrepareAndShowVotingOSD(HWND hwndApp, IVMRWindowlessControl* pWindowlessControl)
{
	isOSDVisible = true;
	SetVotingOSD(hwndApp, pWindowlessControl);
}

/**
Method for setting the voting OSD
\param hwndApp application handle
\param pWindowlessControl windowless control for showing the OSD
*/
HRESULT OSD::SetVotingOSD(HWND hwndApp, IVMRWindowlessControl* pWindowlessControl)
{
    HRESULT hr;
    VMRALPHABITMAP bmpInfo={0};

    // Read the current bitmap settings
    hr = g_pBMP->GetAlphaBitmapParameters(&bmpInfo);
    if (FAILED(hr))
    {
		MessageBox(NULL, "Failed to retrieve AlphaBitmap parameters!", "Error", MB_OK);
		//OnPaint(appWindow);
		return hr;
	}

	hr = BlendVotingSlider(hwndApp, pWindowlessControl, 0);

    return hr;
}

/**
Method for blending the voting slider on the OSD
\param hwndApp application handle
\param pWindowlessControl windowless control for showing the OSD
\param curVote current voting
\return error or ok
*/
HRESULT OSD::BlendVotingSlider(HWND hwndApp, IVMRWindowlessControl* pWindowlessControl, int curVote)
{
    LONG cx, cy;
    HRESULT hr;

    // Read the default video size
	hr = pWindowlessControl->GetNativeVideoSize(&cx, &cy, NULL, NULL);
    if (FAILED(hr))
      return hr;

    // Create a device context compatible with the current window
    HDC hdc = GetDC(hwndApp);
    HDC hdcBmp = CreateCompatibleDC(hdc);

    // voting scale size
	//int scaleHeight = (int)100/cy*5;
	//int scaleWidth = (int)100/cx*5;
	int scaleHeight = 61;
	int scaleWidth = 11;

    // Create a new bitmap that is compatible with the current window
    HBITMAP hbm = CreateCompatibleBitmap(hdc, scaleWidth, scaleHeight);
    ReleaseDC(hwndApp, hdc);

    // Select our bitmap into the device context and save the old one
    BITMAP bm;
    HBITMAP hbmOld;
    GetObject(hbm, sizeof(bm), &bm);
    hbmOld = (HBITMAP)SelectObject(hdcBmp, hbm);

    // Set initial bitmap settings
    RECT rcScale;
	// unable to use fill rect with transparency (do not know why)
	//HBRUSH hBrush = CreateSolidBrush(RGB(255,255,255));
	SetRect(&rcScale, 0, 0, scaleWidth, scaleHeight);
	//FillRect(hdcBmp, &rcScale, hBrush);
	HPEN hPen = CreatePen(PS_SOLID, 1, g_rgbColors);
	
    // Draw the requested text string onto the bitmap
	SelectObject(hdcBmp, hPen);
	
	// draw scale
	MoveToEx(hdcBmp, 0, 0, NULL);
	LineTo(hdcBmp, 11, 0);
    MoveToEx(hdcBmp, 0, 12, NULL);
	LineTo(hdcBmp, 11, 12);
    MoveToEx(hdcBmp, 0, 24, NULL);
	LineTo(hdcBmp, 11, 24);
    MoveToEx(hdcBmp, 0, 36, NULL);
	LineTo(hdcBmp, 11, 36);
	MoveToEx(hdcBmp, 0, 48, NULL);
	LineTo(hdcBmp, 11, 48);
	MoveToEx(hdcBmp, 0, 60, NULL);
	LineTo(hdcBmp, 11, 60);
    MoveToEx(hdcBmp, 5, 0, NULL);
	LineTo(hdcBmp, 5, 60);

	// draw selection
	RECT rcSelection;
	HBRUSH hBrush = CreateSolidBrush(RGB(250,0,0));
	int rectWidth = 11;
	int rectHeight = 11;
	if (curVote != 0)
	{
		switch (curVote)
		{
		case 5:
			SetRect(&rcSelection, 0, 1, rectWidth, rectHeight+1);
			break;
		case 4:
			SetRect(&rcSelection, 0, 13, rectWidth, rectHeight+13);
			break;
		case 3:
			SetRect(&rcSelection, 0, 25, rectWidth, rectHeight+25);
			break;
		case 2:
			SetRect(&rcSelection, 0, 37, rectWidth, rectHeight+37);
			break;
		case 1:
			SetRect(&rcSelection, 0, 49, rectWidth, rectHeight+49);
			break;
		}

		FillRect(hdcBmp, &rcSelection, hBrush);
	}

	DeleteObject(hPen);
	DeleteObject(hBrush);
        
    // Configure the VMR's bitmap structure
    VMRALPHABITMAP bmpInfo;
    ZeroMemory(&bmpInfo, sizeof(bmpInfo) );
    bmpInfo.dwFlags = VMRBITMAP_HDC;
    bmpInfo.hdc = hdcBmp;  // DC which has selected our bitmap

    // Display the bitmap in the bottom right corner.
    // rSrc specifies the source rectangle in the GDI device context 
    // rDest specifies the destination rectangle in composition space (0.0f to 1.0f)
    bmpInfo.rDest.left  =  0.94f;
	bmpInfo.rDest.right = 1.0f-0.01f;
    bmpInfo.rDest.top = (float)(cy - bm.bmHeight) / (float)cy - 0.01f;
    bmpInfo.rDest.bottom = 1.0f-0.01f;
    bmpInfo.rSrc = rcScale;

    // Transparency value 1.0 is opaque, 0.0 is transparent.
    bmpInfo.fAlpha = 1.0f;

    // Set the COLORREF so that the bitmap outline will be transparent
    //SetColorRef(bmpInfo);

    // Give the bitmap to the VMR for display
    hr = g_pBMP->SetAlphaBitmap(&bmpInfo);
    if (FAILED(hr))
	{
		MessageBox(NULL, "Failed to display OSD!", "Error", MB_OK);
		//OnPaint(appWindow);
	}

    // Select the initial objects back into our device context
    DeleteObject(SelectObject(hdcBmp, hbmOld));
    
    // Clean up resources
    DeleteObject(hbm);
    DeleteDC(hdcBmp);

    return hr;
}

/**
Method for refreshing the OSD
\param hwndApp application handle
\param pWindowlessControl windowless control for showing the OSD
\param pMediaPosition media position informations
\param pBasicAudio audio informations
*/
void OSD::RefreshOSD(HWND hwndApp, IVMRWindowlessControl* pWindowlessControl, IMediaPosition* pMediaPosition, IBasicAudio* pBasicAudio)
{
	REFTIME duration = 0;
	if (SUCCEEDED(pMediaPosition->get_Duration(&duration)))
	{
		REFTIME timeNow;
		
		if (SUCCEEDED(pMediaPosition->get_CurrentPosition(&timeNow)))
		{
			double rate = 1.0;
			if (SUCCEEDED(pMediaPosition->get_Rate(&rate)))
			{
				long volume = MAX_VOLUME;
				if (SUCCEEDED(pBasicAudio->get_Volume(&volume)))
				{
					timeNow = Utils::Round(timeNow, 1);
					duration = Utils::Round(duration, 1);
					char msg[MAX_PATH];

					if (volume == MUTE_VOLUME)
						sprintf_s(msg, "Volume: mute, Rate: %.1f, Time: %.1f/%.1f", 
							rate, timeNow, duration);
					else
						sprintf_s(msg, "Volume: %i%%, Rate: %.1f, Time: %.1f/%.1f", 
						Utils::VolumeToPercent(volume), rate, timeNow, duration);
					
					BlendApplicationText(hwndApp, pWindowlessControl, msg);
					return;
				}
			}
		}
	}

	BlendApplicationText(hwndApp, pWindowlessControl, "Unable to determine data!");
}

/**
Method for preparing and showing the OSD
\param hwndApp application handle
\param pWindowlessControl windowless control for showing the OSD
\param pMediaPosition media position informations
\param pBasicAudio audio informations
*/
void OSD::PrepareAndShowOSD(HWND hwndApp, IVMRWindowlessControl* pWindowlessControl, IMediaPosition* pMediaPosition, IBasicAudio* pBasicAudio)
{
	REFTIME duration = 0;
	if (SUCCEEDED(pMediaPosition->get_Duration(&duration)))
	{
		REFTIME timeNow;
		
		if (SUCCEEDED(pMediaPosition->get_CurrentPosition(&timeNow)))
		{
			double rate = 1.0;
			if (SUCCEEDED(pMediaPosition->get_Rate(&rate)))
			{
				long volume = MAX_VOLUME;
				if (SUCCEEDED(pBasicAudio->get_Volume(&volume)))
				{
					timeNow = Utils::Round(timeNow, 1);
					duration = Utils::Round(duration, 1);
					char msg[MAX_PATH];

					if (volume == MUTE_VOLUME)
						sprintf_s(msg, "Volume: mute, Rate: %.1f, Time: %.1f/%.1f", 
							rate, timeNow, duration);
					else
						sprintf_s(msg, "Volume: %i%%, Rate: %.1f, Time: %.1f/%.1f", 
						Utils::VolumeToPercent(volume), rate, timeNow, duration);
					
					SetOSD(hwndApp, pWindowlessControl, msg);
					return;
				}
			}
		}
	}
		
	SetOSD(hwndApp, pWindowlessControl, "Unable to determine data!");
}

/**
Method for setting the OSD
\param hwndApp application handle
\param pWindowlessControl windowless control for showing the OSD
\param szOSDText text to be written on the OSD
\return error or ok
*/
HRESULT OSD::SetOSD(HWND hwndApp, IVMRWindowlessControl* pWindowlessControl, TCHAR* szOSDText = NULL)
{
    HRESULT hr;
    VMRALPHABITMAP bmpInfo={0};

    // Read the current bitmap settings
    hr = g_pBMP->GetAlphaBitmapParameters(&bmpInfo);
    if (FAILED(hr))
    {
		MessageBox(NULL, "Failed to retrieve AlphaBitmap parameters!", "Error", MB_OK);
		//OnPaint(appWindow);
		return hr;
	}

    // If the request is to disable the bitmap, then disable it
    // and ignore the other flags for now
	if (isOSDVisible)
    {
        // Temporarily disable bitmap display
        bmpInfo.dwFlags = VMRBITMAP_DISABLE;

        // Update the bitmap settings
        hr = g_pBMP->UpdateAlphaBitmapParameters(&bmpInfo);
        if (FAILED(hr))
		{
			MessageBox(NULL, "Failed to disable OSD!", "Error", MB_OK);
			//OnPaint(appWindow);
		}
		isOSDVisible = false;
    }
    else
    {
		hr = BlendApplicationText(hwndApp, pWindowlessControl, szOSDText);
		isOSDVisible = true;
    }

    return hr;
}


/**
Method for setting the text font
\param hwndApp application handle
\return create font object
*/
HFONT OSD::SetTextFont(HWND appWindow) 
{ 
    CHOOSEFONT cf={0}; 
    LOGFONT lf={0}; 
    HFONT hfont; 
    HDC hdc;
    LONG lHeight;

    // Convert requested font point size to logical units
	hdc = GetDC(appWindow );
    lHeight = -MulDiv( g_lFontPointSize, GetDeviceCaps(hdc, LOGPIXELSY), 72 );
    ReleaseDC( appWindow, hdc );

    // Initialize members of the LOGFONT structure. 
    StringCchCopy(lf.lfFaceName, LF_FACESIZE, g_szFontName);
    lf.lfHeight = lHeight;      // Logical units

    // Prevent font smoothing, which could distort the text and leave
    // white pixels on the edges.  Disabling antialiasing leads to 
    // smoother text in this context.
    lf.lfQuality = NONANTIALIASED_QUALITY;

    // Initialize members of the CHOOSEFONT structure. 
    cf.lStructSize = sizeof(CHOOSEFONT); 
	cf.hwndOwner   = appWindow; 
    cf.hDC         = (HDC)NULL; 
    cf.lpLogFont   = &lf; 
    cf.iPointSize  = g_lFontPointSize * 10; 
    cf.rgbColors   = g_rgbColors; 
    cf.lCustData   = 0L; 
    cf.lpfnHook    = (LPCFHOOKPROC)NULL; 
    cf.hInstance   = (HINSTANCE) NULL; 
    cf.lpszStyle   = g_szFontStyle; 
    cf.nFontType   = SCREEN_FONTTYPE; 
    cf.nSizeMin    = 0; 
    cf.lpTemplateName = NULL; 
    cf.Flags = CF_SCREENFONTS | CF_SCALABLEONLY | CF_INITTOLOGFONTSTRUCT | 
               CF_EFFECTS     | CF_USESTYLE     | CF_LIMITSIZE; 

    // Limit font size to prevent bitmap from becoming too wide
    cf.nSizeMax = 30;  
    cf.rgbColors = g_rgbColors;

    // Save the user's selections for configuring the dialog box next time.
    // The style is automatically saved in g_szFontStyle (cf.lpszStyle)
    (void)StringCchCopy(g_szFontName, NUMELMS(g_szFontName), lf.lfFaceName);
 
	// Create a logical font based on the user's selection and 
    // return a handle identifying that font.  
    hfont = CreateFontIndirect(cf.lpLogFont); 
    return (hfont); 
}

/**
Method for setting the color for the OSD
\param bmpInfo bitmap info to be updated
*/
void OSD::SetColorRef(VMRALPHABITMAP& bmpInfo)
{
    // Set the COLORREF so that the bitmap outline will be transparent
    bmpInfo.clrSrcKey = RGB(255, 255, 255);  // Pure white
    bmpInfo.dwFlags |= VMRBITMAP_SRCCOLORKEY;
}

/**
Method for for blending the text on the OSD
\param hwndApp application handle
\param pWindowlessControl windowless control for showing the OSD
\param szNewText new text to be placed on the OSD
\return error or ok
*/
HRESULT OSD::BlendApplicationText(HWND hwndApp, IVMRWindowlessControl* pWindowlessControl, TCHAR *szNewText)
{
    LONG cx, cy;
    HRESULT hr;

    // Read the default video size
	hr = pWindowlessControl->GetNativeVideoSize(&cx, &cy, NULL, NULL);
    if (FAILED(hr))
      return hr;

    // Create a device context compatible with the current window
    HDC hdc = GetDC(hwndApp);
    HDC hdcBmp = CreateCompatibleDC(hdc);

    // Write with a known font by selecting it into our HDC
    HFONT hOldFont = (HFONT) SelectObject(hdcBmp, SetTextFont(hwndApp));

    // Determine the length of the string, then determine the
    // dimensions (in pixels) of the character string using the
    // currently selected font.  These dimensions are used to create
    // a bitmap below.
    int nLength, nTextBmpWidth, nTextBmpHeight;
    SIZE sz={0};
    nLength = (int) _tcslen(szNewText);
    GetTextExtentPoint32(hdcBmp, szNewText, nLength, &sz);
    nTextBmpHeight = sz.cy;
    nTextBmpWidth  = sz.cx;

    // Create a new bitmap that is compatible with the current window
    HBITMAP hbm = CreateCompatibleBitmap(hdc, nTextBmpWidth, nTextBmpHeight);
    ReleaseDC(hwndApp, hdc);

    // Select our bitmap into the device context and save the old one
    BITMAP bm;
    HBITMAP hbmOld;
    GetObject(hbm, sizeof(bm), &bm);
    hbmOld = (HBITMAP)SelectObject(hdcBmp, hbm);

    // Set initial bitmap settings
    RECT rcText;
    SetRect(&rcText, 0, 0, nTextBmpWidth, nTextBmpHeight);
    SetBkColor(hdcBmp, RGB(255, 255, 255)); // Pure white background
    SetTextColor(hdcBmp, g_rgbColors);      // Write text with requested color

    // Draw the requested text string onto the bitmap
    TextOut(hdcBmp, 0, 0, szNewText, nLength);

    // Configure the VMR's bitmap structure
    VMRALPHABITMAP bmpInfo;
    ZeroMemory(&bmpInfo, sizeof(bmpInfo) );
    bmpInfo.dwFlags = VMRBITMAP_HDC;
    bmpInfo.hdc = hdcBmp;  // DC which has selected our bitmap

    // Display the bitmap in the bottom left corner.
    // rSrc specifies the source rectangle in the GDI device context 
    // rDest specifies the destination rectangle in composition space (0.0f to 1.0f)
    bmpInfo.rDest.left  =  0.0f;
    bmpInfo.rDest.right = 0.0f + bm.bmWidth/(float)cx;
    bmpInfo.rDest.top = (float)(cy - bm.bmHeight) / (float)cy - 0.04f;
    bmpInfo.rDest.bottom = 1.0f - 0.04f;
    bmpInfo.rSrc = rcText;

    // Transparency value 1.0 is opaque, 0.0 is transparent.
    bmpInfo.fAlpha = 1.0f;

    // Set the COLORREF so that the bitmap outline will be transparent
    SetColorRef(bmpInfo);

    // Give the bitmap to the VMR for display
    hr = g_pBMP->SetAlphaBitmap(&bmpInfo);
    if (FAILED(hr))
	{
		MessageBox(NULL, "Failed to display OSD!", "Error", MB_OK);
		//OnPaint(appWindow);
	}

    // Select the initial objects back into our device context
    DeleteObject(SelectObject(hdcBmp, hbmOld));
    SelectObject(hdc, hOldFont);

    // Clean up resources
    DeleteObject(hbm);
    DeleteDC(hdcBmp);

    return hr;
}
