/*
 * This file is part of SEMP.
 *
 * SEMP is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SEMP is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SEMP. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2009, 2010, 2011, 2012 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */

#include "../stdafx.h"
#include "typedef.h"
#include "EffectManager.h"
#include <math.h>

#if __USEAMBX__
	#include "amBXErrorHandler.h"
#endif

#define EFFECT_TIME_SPAN					0.15f
#define EFFECT_TIME_SPAN_ERROR_CORRECTION	0.05f
#define EFFECT_TIME_SPAN_MAX				0.3f	// 300ms is the maximum we would look back for effects
#define DEFAULT_LIGHTFADE_TIME	30	// 30ms fade between light effects

#if __USEAMBX__
#define RUMBLE_MIN_UPDATE_INTERVAL			30		// update interval of the rumble in ms
#define LIGHT_MIN_UPDATE_INTERVAL			30
#define FAN_MIN_UPDATE_INTERVAL				30
#endif

#if __USEVORTEX__
#define SCENT_CONFIG_FILE	"scentconfig.dat"
#define VORTEXACTIVE_CODEPAGE	30
#define VORTEXACTIVE_FAN1	2
#define VORTEXACTIVE_FAN2	4
#define VORTEXACTIVE_FAN3	8
#define VORTEXACTIVE_FAN4	16
#endif

/**
Constructor
\param disableFateTime disables the default fading time between effects
\param scentConfiguration path to the scent configuration file
*/
EffectManager::EffectManager(bool disableFateTime, std::string scentConfiguration)
{
	bDisableFateTime = disableFateTime;
	Initialisation(scentConfiguration);
}

/**
Constructor
\param disableFateTime disables the default fading time between effects
\param elements vector containing all sensory effects for the multimedia file
\param scentConfiguration path to the scent configuration file
*/
EffectManager::EffectManager(bool disableFateTime, std::vector<Element*> elements, std::string scentConfiguration)
{
	bDisableFateTime = disableFateTime;
	Initialisation(scentConfiguration);
	m_Elements = elements;	
}

/**
Method for initialising the effect manager
\param scentConfiguration path to the scent configuration file
*/
void EffectManager::Initialisation(std::string scentConfiguration)
{	
	//logger = new Logger("effectManager.log");
	effectMutex = CreateMutex(NULL, FALSE, NULL);
	globalLock = CreateMutex(NULL, FALSE, NULL);		// this global lock prevents changes during playEffectAtTime

	error = SEMP_OK;

#if __USEAMBX__
	lightContainer = new LightContainer();
	fanContainer = new FanContainer();
	rumbleContainer = new RumbleContainer();
#endif

#if __USEVORTEX__
	scentContainer = new ScentContainer();
#endif

	zeroTime = false;
	devChanged = false;
	videoLength = 0.0;

	lightAutoColor = false;
	oldLightAutoColor = false;
	i_mtime = 0.0;
	i_stime = 0.0;
	add_tspan = 0.0;

#if __USEAMBX__
	// create amBX instance
	pIamBX = NULL;
	error = amBXCreateInterface(&pIamBX, 1, 0, "SEMP amBX plugin", "0.11", NULL, 0);

	if (error == amBX_OK)
	{
		DetectAttachedDevices();

		this->Update_amBX();
	}
#endif

#if __USEVORTEX__
	//TODO: error checking
	if (scentConfiguration.empty())
		scentConfiguration = SCENT_CONFIG_FILE;

	InitializeVortexActive(scentConfiguration);
	ActivateAllScentDeviceFans(scentContainer, false);
		
#endif
}

/**
Destructor
*/
EffectManager::~EffectManager()
{
#if __USEAMBX__
	ActivateAllAvailableDevices(false);
	
	if (pIamBX != NULL)
		pIamBX->release(pIamBX); // automatically releases all IamBX_* interfaces

		if (lightContainer != NULL)
	{
		// free all Light classes
		for (int i = 0; i < amBXLocationMap.GetEnumLocationsLength(); i++)
		{
			Light *light = lightContainer->GetLight((amBX_loc)amBXLocationMap.GetEnumLocations()[i]);
			if (light != NULL)
			{
				delete light;
				light = NULL;
			}
		}

		delete lightContainer;
		lightContainer = NULL;
	}

	if (fanContainer != NULL)
	{
		// free all Fan classes
		for (int i = 0; i < amBXLocationMap.GetEnumLocationsLength(); i++)
		{
			Fan *fan = fanContainer->GetFan((amBX_loc)amBXLocationMap.GetEnumLocations()[i]);
			if (fan != NULL)
			{
				delete fan;
				fan = NULL;
			}
		}

		delete fanContainer;
		fanContainer = NULL;
	}

	if (rumbleContainer != NULL)
	{
		// free all Rumble classes
		for (int i = 0; i < amBXLocationMap.GetEnumLocationsLength(); i++)
		{
			Rumble *rumble = rumbleContainer->GetRumble((amBX_loc)amBXLocationMap.GetEnumLocations()[i]);
			if (rumble != NULL)
			{
				delete rumble;
				rumble = NULL;
			}
		}
		delete rumbleContainer;
		rumbleContainer = NULL;
	}
#endif

#if __USEVORTEX__
	// free scent devices
	if (scentContainer != NULL)
	{
		// free all Scent classes
		std::vector<Scent*> allScents = scentContainer->GetAllScents();
		for (unsigned int i = 0; i < allScents.size(); i++)
		{
			Scent *scent = allScents.at(i);
			if (scent != NULL)
			{
				delete scent;
				scent = NULL;
			}
		}
		
		// free vortexactive
		if (scentContainer->IsDeviceActive())
		{
			ActivateAllScentDeviceFans(scentContainer, false);
			CloseScentDevice(scentContainer);
		}

		delete scentContainer;
		scentContainer = NULL;
	}
#endif

	for (unsigned int i = 0; i < m_Elements.size(); i++)
	{
		delete m_Elements.at(i);
		m_Elements.at(i) = NULL;
	}

	CloseHandle(effectMutex);
	CloseHandle(globalLock);
}

#if __USEAMBX__
/**
Method for updating the amBX system
*/
void EffectManager::Update_amBX()
{
	WaitForSingleObject(effectMutex,INFINITE);
	error = pIamBX->update(pIamBX,NULL);
	ReleaseMutex(effectMutex);
}
#endif

/**
Method for setting the video length
\param length of the video
*/
void EffectManager::SetVideoLength(double length)
{
	videoLength = length;
}

/**
Getter for the video length
\return video length
*/
double EffectManager::GetVideoLength()
{
	return videoLength;
}

/**
Method for playing sensory effects
\param e element to be played
\return false if an error occurred else true
*/
bool EffectManager::PlayEffect(Element e)
{
	//e.print();
	switch (e.GetType())
	{
	case TYPE_FAN:
		HandleWind(e);
		break;
	case TYPE_VIBRATOR:
		HandleVibration(e);
		break;
	case TYPE_LIGHT:
		HandleLight(e);
		break;
	case TYPE_WATERSPRAYER:
	case TYPE_AIRCONDITIONER:
	case TYPE_FOG:
		// not needed/used
		break;
	case TYPE_PERFUMER:
		HandleScent(e);
		break;
	case TYPE_NONE:
	default:
		return false;
	}

	/*char msg[100];
	sprintf(msg, "type: %i; time: %.1f", e.getType(), e.getTime());*/

	if (error != SEMP_OK)
		return false;
	else
		return true;
}

/**
Method for adding an effect to the effect manager
\param e element to be added
*/
void EffectManager::AddEffect(Element *e)
{
	if (e != NULL)
		m_Elements.push_back(e);

	std::sort(m_Elements.begin(), m_Elements.end(), ElementCompare);
}

/**
Compare function for elements
\param element1 first element
\param element2 second element
\return true if time of first element is smaller then of the second
*/
bool EffectManager::ElementCompare(const Element* element1, const Element* element2)
{
	return element1->GetTime() < element2->GetTime();
}

/**
Method for adding a vector of elements
\param e vector with elements
*/
void EffectManager::AddEffects(std::vector<Element *> e)
{
	for (unsigned int i = 0; i < e.size(); i++)
		m_Elements.push_back(e.at(i));

	std::sort(m_Elements.begin(), m_Elements.end(), ElementCompare);
}

/**
Method for playing effects at a given time
\param time at which the effect should be played
\return true if played else false
*/
bool EffectManager::PlayEffectAtTime(double time)
{
	WaitForSingleObject(globalLock, INFINITE); // do not let anyone else change anything during this phase

	bool played = false;

	// check when we were called last, if this intervall is bigger than our 100ms then try to adjust the time span
	// if the current timestamp is lower than the last, reset 
	if(time < i_mtime || !zeroTime)
	{
		add_tspan = 0.0;
	}
	else
	{
		// calc the missing portion
		if(time - i_mtime > EFFECT_TIME_SPAN) 
		{
				add_tspan = (time - i_mtime) - EFFECT_TIME_SPAN + EFFECT_TIME_SPAN_ERROR_CORRECTION;	// add a little error correction, so we can be sure that we don't miss any effect
				// in worst case scenarios we may get here seconds ...
				// prevent this by checking for a maximum additional time spam
				if(add_tspan+EFFECT_TIME_SPAN > EFFECT_TIME_SPAN_MAX) 
					add_tspan = EFFECT_TIME_SPAN_MAX - EFFECT_TIME_SPAN;
		}
		
		if(time - i_mtime <= EFFECT_TIME_SPAN) 
			add_tspan = 0.0;	// we are within the time span of 100ms, we won't need to check further backwards
	}
	
	for (std::vector<Element*>::const_iterator it = m_Elements.begin(); it != m_Elements.end(); ++it)
	{
		// if we rewind the movie and the effects have already been played, we have to reset their timestamp
		if(time < (*it)->GetPlayedTime())
			(*it)->SetPlayedTime(-1.0);

		// do we have played the effect already ?		
		if((*it)->GetPlayedTime() < 0.0f)
		{
			if ((*it)->GetTime() == 0.0 && !zeroTime)
			{
				(*it)->SetPlayedTime(time);
				played = PlayEffect(*(*it));
			}
			else if ((*it)->GetTime() >= time-(EFFECT_TIME_SPAN+add_tspan) && (*it)->GetTime() <= time)
			{
				(*it)->SetPlayedTime(time);
				played = PlayEffect(*(*it));	
			}
		}
	}
	i_mtime = time;		// save our timestamp
	zeroTime = true;

#if __USEAMBX__
	// play fade
	for (int i = 0; i < amBXLocationMap.GetEnumLocationsLength(); i++)
	{
		Rumble *rumble = rumbleContainer->GetRumble((amBX_loc)amBXLocationMap.GetEnumLocations()[i]);
		if (rumble->IsFadeActive())
			FadeVibration(time, rumble);
	}
	
	for (int i = 0; i < amBXLocationMap.GetEnumLocationsLength(); i++)
	{
		Fan *fan = fanContainer->GetFan((amBX_loc)amBXLocationMap.GetEnumLocations()[i]);
		if (fan->IsFadeActive())
			FadeWind(time, fan);
	}		

	if (devChanged)
		this->Update_amBX();
#endif

	devChanged = false;

	ReleaseMutex(globalLock);

	return played;
}

/**
Method for determining if there was an error
\return true if an error occurred else false
*/
bool EffectManager::HasError()
{
	bool retVal = false;

	if (error != SEMP_OK)
		retVal = true;

	return retVal;
}

/**
Method for clearing the error state
*/
void EffectManager::ClearError()
{
	error = SEMP_OK;
}

/**
Method for getting the error string
\return string with the error message
*/
std::string EffectManager::GetErrorString()
{
#if __USEAMBX__
	return amBXErrorHandler::GetErrorMessage((amBX_RESULT)error);
#else
	return "";
#endif
}

/**
Method for handling wind effects
\param e element to be handled
*/
void EffectManager::HandleWind(Element e)
{
#if __USEAMBX__
	if (strcmp(e.GetValue(ATT_GEN_ACTIVATE).c_str(), SEMP_TRUE) == 0)
	{
		std::vector<std::string> range;

		if (!strcmp(e.GetValue(ATT_INTENSITYRANGE).c_str(), "") == 0)
		{
			range = Utils::StringSplit(e.GetValue(ATT_INTENSITYRANGE), " ");
			if (!IsInIntensityRange(atof(e.GetValue(ATT_INTENSITYVALUE).c_str()), range))
				return;
		}

		double fanPTS = e.GetPlayedTime();	// when was our effect played
		double fanFade = atof(e.GetValue(ATT_GEN_FADE).c_str());
		double fanDuration = atof(e.GetValue(ATT_GEN_DURATION).c_str());
		double fanIntensity = atof(e.GetValue(ATT_INTENSITYVALUE).c_str());

		// calculate the percentage of intensity
		if (range.size() == 2)
		{
			double upperBound = atof(range.at(1).c_str());
			fanIntensity = (100.0 / upperBound) * fanIntensity;
		}

		// check if fade is not bigger than duration
		if (fanFade != 0 && fanDuration != 0)
		{
			if (fanFade > fanDuration) 
			{
				printf("Fade cannot be bigger than the duration of effect! Aborting!");
				return;
			}
		}

		if (strcmp(e.GetValue(ATT_GEN_LOCATION).c_str(), "") != 0)
		{
			std::vector<amBX_loc> amBXLocations = amBXLocationMap.GetAmBXLocations(e.GetValue(ATT_GEN_LOCATION).c_str());
			if (amBXLocations.size() > 0)
			{
				for (unsigned int i = 0; i < amBXLocations.size(); i++) 
				{
					Fan *fan = fanContainer->GetFan(amBXLocations.at(i));
					fan->SetFadeTime(fanFade);
					fan->SetDuration(fanDuration);
					fan->SetIntensity(fanIntensity);
					fan->SetPTS(fanPTS);

					// calculate stepping
					if (fanFade > 0) 
					{
						fan->ActivateFade(true);
						if (fan->GetOldIntensityValue() > 0)
						{
							fan->SetEffectDifference(fan->GetIntensity()-fan->GetOldIntensityValue());
							fan->SetIntermediateIntensity(fan->GetOldIntensityValue());
						}
						else
						{
							fan->SetIntermediateIntensity(0.0);
							fan->SetEffectDifference(fan->GetIntensity());
						}
					}
					else
						fan->ActivateFade(false);	

					fan->SetOldIntensityValue(fan->GetIntensity());

					if (!fan->IsFadeActive())
						fan->SetIntensityToDevice((float)fan->GetIntensity());

					devChanged = true;
				}
			}
		}
		else
		{
			// get all location
			// activate all
			std::vector<amBX_loc> amBXLocations = amBXLocationMap.GetAmBXLocations(Classification::LOCATIONURIBASE + Classification::LOC_EVERYWHERE + Classification::LOC_EVERYWHERE + Classification::LOC_EVERYWHERE);
			if (amBXLocations.size() > 0)
			{
				for (unsigned int i = 0; i < amBXLocations.size(); i++) 
				{
					Fan *fan = fanContainer->GetFan(amBXLocations.at(i));
					fan->SetFadeTime(fanFade);
					fan->SetDuration(fanDuration);
					fan->SetIntensity(fanIntensity);
					fan->SetPTS(fanPTS);

					// calculate stepping
					if (fanFade > 0) 
					{
						fan->ActivateFade(true);
						if (fan->GetOldIntensityValue() > 0)
						{
							fan->SetEffectDifference(fan->GetIntensity()-fan->GetOldIntensityValue());
							fan->SetIntermediateIntensity(fan->GetOldIntensityValue());
						}
						else
						{
							fan->SetIntermediateIntensity(0.0);
							fan->SetEffectDifference(fan->GetIntensity());
						}
					}
					else
						fan->ActivateFade(false);

					fan->SetOldIntensityValue(fan->GetIntensity());

					if (!fan->IsFadeActive())
						fan->SetIntensityToDevice((float)fan->GetIntensity());

					devChanged = true;
				}
			}
		}
	}
	else if (strcmp(e.GetValue(ATT_GEN_ACTIVATE).c_str(), SEMP_FALSE) == 0)
	{
		if (strcmp(e.GetValue(ATT_GEN_LOCATION).c_str(), "") != 0)
		{
			std::vector<amBX_loc> amBXLocations = amBXLocationMap.GetAmBXLocations(e.GetValue(ATT_GEN_LOCATION).c_str());
			if (amBXLocations.size() > 0)
			{
				for (unsigned int i = 0; i < amBXLocations.size(); i++) 
				{
					Fan *fan = fanContainer->GetFan(amBXLocations.at(i));
					error = fan->SetIntensityToDevice(0.0f);
					fan->ActivateFade(false);					
					devChanged = true;
				}
			}
		}
		else
		{
			// deactivate all
			StopAllFans();
			devChanged = true;
		}
	}
#endif
}

/**
Method for handling vibration effects
\param e element to be handled
*/
void EffectManager::HandleVibration(Element e)
{
#if __USEAMBX__
	if (strcmp(e.GetValue(ATT_GEN_ACTIVATE).c_str(), SEMP_TRUE) == 0)
	{
		std::vector<std::string> range;

		if (!strcmp(e.GetValue(ATT_INTENSITYRANGE).c_str(), "") == 0)
		{
			range = Utils::StringSplit(e.GetValue(ATT_INTENSITYRANGE), " ");
			if (!IsInIntensityRange(atof(e.GetValue(ATT_INTENSITYVALUE).c_str()), range))
				return;
		}

		double rumblePTS = e.GetPlayedTime();	// when was our effect played
		double rumbleFade = atof(e.GetValue(ATT_GEN_FADE).c_str());
		double rumbleDuration = atof(e.GetValue(ATT_GEN_DURATION).c_str());
		double rumbleIntensity = atof(e.GetValue(ATT_INTENSITYVALUE).c_str());

		// calculate the percentage of intensity
		if (range.size() == 2)
		{
			double upperBound = atof(range.at(1).c_str());
			rumbleIntensity = (100.0 / upperBound) * rumbleIntensity;
		}

		// check if fade is not bigger than duration
		if (rumbleFade != 0 && rumbleDuration != 0)
		{
			if (rumbleFade > rumbleDuration) 
			{
				printf("Fade cannot be bigger than the duration of effect! Aborting!");
				return;
			}
		}

		if (strcmp(e.GetValue(ATT_GEN_LOCATION).c_str(), "") != 0)
		{
			std::vector<amBX_loc> amBXLocations = amBXLocationMap.GetAmBXLocations(e.GetValue(ATT_GEN_LOCATION).c_str());
			if (amBXLocations.size() > 0)
			{
				for (unsigned int i = 0; i < amBXLocations.size(); i++) 
				{
					Rumble *rumble = rumbleContainer->GetRumble(amBXLocations.at(i));
					rumble->SetFadeTime(rumbleFade);
					rumble->SetDuration(rumbleDuration);
					rumble->SetIntensity(rumbleIntensity);
					rumble->SetPTS(rumblePTS);

					// calculate stepping
					if (rumbleFade > 0) 
					{
						rumble->ActivateFade(true);
						if (rumble->GetOldIntensityValue() > 0)
						{
							rumble->SetEffectDifference(rumble->GetIntensity()-rumble->GetOldIntensityValue());
							rumble->SetIntermediateIntensity(rumble->GetOldIntensityValue());
						}
						else
						{
							rumble->SetIntermediateIntensity(0.0);
							rumble->SetEffectDifference(rumble->GetIntensity());
						}
					}
					else
						rumble->ActivateFade(false);

					rumble->SetOldIntensityValue(rumble->GetIntensity());

					if (!rumble->IsFadeActive())
						rumble->SetIntensityToDevice((float)rumble->GetIntensity());

					devChanged = true;
				}
			}
		}
		else
		{
			// get all location
			// activate all
			std::vector<amBX_loc> amBXLocations = amBXLocationMap.GetAmBXLocations(Classification::LOCATIONURIBASE + Classification::LOC_EVERYWHERE + Classification::LOC_EVERYWHERE + Classification::LOC_EVERYWHERE);
			if (amBXLocations.size() > 0)
			{
				for (unsigned int i = 0; i < amBXLocations.size(); i++) 
				{
					Rumble *rumble = rumbleContainer->GetRumble(amBXLocations.at(i));
					rumble->SetFadeTime(rumbleFade);
					rumble->SetDuration(rumbleDuration);
					rumble->SetIntensity(rumbleIntensity);
					rumble->SetPTS(rumblePTS);

					// calculate stepping
					if (rumbleFade > 0) 
					{
						rumble->ActivateFade(true);
						if (rumble->GetOldIntensityValue() > 0)
						{
							rumble->SetEffectDifference(rumble->GetIntensity()-rumble->GetOldIntensityValue());
							rumble->SetIntermediateIntensity(rumble->GetOldIntensityValue());
						}
						else
						{
							rumble->SetIntermediateIntensity(0.0);
							rumble->SetEffectDifference(rumble->GetIntensity());
						}
					}
					else
						rumble->ActivateFade(false);					

					rumble->SetOldIntensityValue(rumble->GetIntensity());

					if (!rumble->IsFadeActive())
						rumble->SetIntensityToDevice((float)rumble->GetIntensity());

					devChanged = true;
				}
			}
		}
	}
	else if (strcmp(e.GetValue(ATT_GEN_ACTIVATE).c_str(), SEMP_FALSE) == 0)
	{
		if (strcmp(e.GetValue(ATT_GEN_LOCATION).c_str(), "") != 0)
		{
			std::vector<amBX_loc> amBXLocations = amBXLocationMap.GetAmBXLocations(e.GetValue(ATT_GEN_LOCATION).c_str());
			if (amBXLocations.size() > 0)
			{
				for (unsigned int i = 0; i < amBXLocations.size(); i++) 
				{
					Rumble *rumble = rumbleContainer->GetRumble(amBXLocations.at(i));
					error = rumble->SetIntensityToDevice(0.0f);
					rumble->ActivateFade(false);					
					devChanged = true;
				}
			}
		}
		else
		{
			// deactivate all
			StopAllRumblers();
			devChanged = true;
		}
	}
#endif
}

/**
Method for handling light effects
\param e element to be handled
*/
void EffectManager::HandleLight(Element e)
{
#if __USEAMBX__
	if (strcmp(e.GetValue(ATT_GEN_ACTIVATE).c_str(), SEMP_TRUE) == 0)
	{
		// NOTE: if autocolor is activated everything else will be ignored
		if (strcmp(e.GetValue(ATT_GEN_AUTOEXTRACTION).c_str(), VISUAL) == 0 ||
			strcmp(e.GetValue(ATT_GEN_AUTOEXTRACTION).c_str(), BOTH) == 0)
		{
			lightAutoColor = true;
			oldLightAutoColor = true;
			DisplayColor();
		}
		else
		{
			// check if intensity value is in range
			std::vector<std::string> range;

			if (!strcmp(e.GetValue(ATT_INTENSITYRANGE).c_str(), "") == 0)
			{
				range = Utils::StringSplit(e.GetValue(ATT_INTENSITYRANGE), " ");
				if (!IsInIntensityRange(atof(e.GetValue(ATT_INTENSITYVALUE).c_str()), range))
					return;
			}

			double lightPTS = e.GetPlayedTime();
			double lightFade = atof(e.GetValue(ATT_GEN_FADE).c_str());
			double lightDuration = atof(e.GetValue(ATT_GEN_DURATION).c_str());
			double lightIntensity = atof(e.GetValue(ATT_INTENSITYVALUE).c_str());

			// calculate the percentage of intensity
			if (range.size() == 2)
			{
				double upperBound = atof(range.at(1).c_str());
				lightIntensity = (100.0 / upperBound) * lightIntensity;
			}

			// check if fade is not bigger than duration
			if (lightFade != 0 && lightDuration != 0)
			{
				if (lightFade > lightDuration) 
				{
					printf("Fade cannot be bigger than the duration of effect! Aborting!");
					return;
				}
			}
			
			COLORREF color = NULL;

			if (strcmp(e.GetValue(ATT_LIGHT_COLOR).c_str(), "") != 0)
			{
				std::string col = e.GetValue(ATT_LIGHT_COLOR);
				// check if we have a color starting with #
				// does not mean that it is automatically hex!
				if (col.at(0) == '#')
					col = col.erase(0, 1);

				if (Utils::IsHex(col))
				{
					color = strtol(col.c_str(), NULL, 16);
					// need to switch from BGR to RGB because wrong storing of data in COLORREF
					color = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
				}
				else
					color = classification.GetRGBforColor(col);

				if (lightIntensity > 0.0)
				{
					int r = (int)Utils::Round(GetRValue(color) * (lightIntensity/100));
					int g = (int)Utils::Round(GetGValue(color) * (lightIntensity/100));
					int b = (int)Utils::Round(GetBValue(color) * (lightIntensity/100));

					r = Utils::Clamp(r, 0, 255);
					g = Utils::Clamp(g, 0, 255);
					b = Utils::Clamp(b, 0, 255);
					
					color = RGB(r,g,b);
				}
			}

			// location
			if (strcmp(e.GetValue(ATT_GEN_LOCATION).c_str(), "") != 0) 
			{
				std::vector<amBX_loc> amBXLocations = amBXLocationMap.GetAmBXLocations(e.GetValue(ATT_GEN_LOCATION).c_str());
				if (amBXLocations.size() > 0)
				{
					for (unsigned int i = 0; i < amBXLocations.size(); i++) 
					{
						Light *light = lightContainer->GetLight(amBXLocations.at(i));
						light->SetFadeTime(lightFade);
						light->SetDuration(lightDuration);
						light->SetIntensity(lightIntensity);
						light->SetPTS(lightPTS);

						if (lightFade > 0) 
						{						
							light->ActivateFade(true);
							error = light->GetLight()->setFadeTime(light->GetLight(), (amBX_time)lightFade);
						}
						else
						{
							light->ActivateFade(false);
							if (bDisableFateTime)
								error = light->GetLight()->setFadeTime(light->GetLight(), 0);
							else
								error = light->GetLight()->setFadeTime(light->GetLight(), DEFAULT_LIGHTFADE_TIME);
						}	

						light->SetOldIntensityValue(light->GetIntensity());

						if (color != NULL)
							light->SetLightColor(GetRValue(color), GetGValue(color), GetBValue(color));
					}
				}
			}
			else
			{
				// get all location
				// activate all
				std::vector<amBX_loc> amBXLocations = amBXLocationMap.GetAmBXLocations(Classification::LOCATIONURIBASE + Classification::LOC_EVERYWHERE + Classification::LOC_EVERYWHERE + Classification::LOC_EVERYWHERE);
				if (amBXLocations.size() > 0)
				{
					for (unsigned int i = 0; i < amBXLocations.size(); i++) 
					{
						Light *light = lightContainer->GetLight(amBXLocations.at(i));
						light->SetFadeTime(lightFade);
						light->SetDuration(lightDuration);
						light->SetIntensity(lightIntensity);
						light->SetPTS(lightPTS);

						if (lightFade > 0) 
						{						
							light->ActivateFade(true);
							error = light->GetLight()->setFadeTime(light->GetLight(), (amBX_time)lightFade);
						}
						else
						{
							light->ActivateFade(false);
							if (bDisableFateTime)
								error = light->GetLight()->setFadeTime(light->GetLight(), 0);
							else
								error = light->GetLight()->setFadeTime(light->GetLight(), DEFAULT_LIGHTFADE_TIME);
						}

						light->SetOldIntensityValue(light->GetIntensity());

						if (color != NULL)
							light->SetLightColor(GetRValue(color), GetGValue(color), GetBValue(color));
					}
				}
			}
			DisplayColor();
		}
	}
	else if (strcmp(e.GetValue(ATT_GEN_ACTIVATE).c_str(), SEMP_FALSE) == 0)
	{
		if (strcmp(e.GetValue(ATT_GEN_LOCATION).c_str(), "") != 0) 
		{
			std::vector<amBX_loc> amBXLocations = amBXLocationMap.GetAmBXLocations(e.GetValue(ATT_GEN_LOCATION).c_str());
			if (amBXLocations.size() > 0)
			{
				for (unsigned int i = 0; i < amBXLocations.size(); i++) 
				{
					Light *light = lightContainer->GetLight(amBXLocations.at(i));
					light->ActivateFade(false);
					light->ResetLight();

					if (bDisableFateTime)
						error = light->GetLight()->setFadeTime(light->GetLight(), 0);
					else
						error = light->GetLight()->setFadeTime(light->GetLight(), DEFAULT_LIGHTFADE_TIME);

					if (error == amBX_OK)
						error = light->GetLight()->setCol(light->GetLight(), 0.0f, 0.0f, 0.0f);
				}
			}
		}
		else
			ActivateAllLights(false);

		devChanged = true;
	}
#endif
}

/**
Method for retrieving the number of effects
\return number of effects
*/
int EffectManager::GetItems()
{
	return m_Elements.size();
}

/**
Method for displaying the color on the light devices
*/
void EffectManager::DisplayColor()
{
#if __USEAMBX__
	// cycle over all available lights and update their displayed color
	std::vector<Light *> allLights = lightContainer->GetAllLights();
	if (allLights.size() > 0)
	{
		for (unsigned int i = 0; i < allLights.size(); i++)
		{
			Light *light = allLights.at(i);
			int *rgb = light->GetLightColor();

			float fRed = CalculatePercent(rgb[0]);
			float fGreen = CalculatePercent(rgb[1]);
			float fBlue = CalculatePercent(rgb[2]);

			error = light->GetLight()->setCol(light->GetLight(), fRed, fGreen, fBlue);
		}
	}

	devChanged = true;
#endif
}

/**
Method for activating all light devices
\param activate true for activating else false for deactivate
*/
void EffectManager::ActivateAllLights(bool activate)
{
#if __USEAMBX__
	std::vector<Light *> allLights = lightContainer->GetAllLights();
	if (allLights.size() > 0)
	{
		for (unsigned int i = 0; i < allLights.size(); i++) 
		{
			Light *light = allLights.at(i);
			
			if (activate)
			{
				int *rgb = light->GetLightColor();
				float fRed = CalculatePercent(rgb[0]);
				float fGreen = CalculatePercent(rgb[1]);
				float fBlue = CalculatePercent(rgb[2]);

				error = light->GetLight()->setCol(light->GetLight(), fRed, fGreen, fBlue);
				if (oldLightAutoColor)
					lightAutoColor = true;
			}
			else
			{				
				if (lightAutoColor)
					lightAutoColor = false;

				// we are setting the light to black and not switch it off
				// else an error can occur
				error = light->GetLight()->setCol(light->GetLight(), 0.0f, 0.0f, 0.0f);
			}
		}
	}

	devChanged = true;
#endif
}

/**
Method for activating all fan devices
\param activate true for activating else false for deactivate
*/
void EffectManager::ActivateAllFans(bool activate)
{
#if __USEAMBX__
	WaitForSingleObject(globalLock,INFINITE);	// wait until the current effect changes are made and then disable the appropriate effekt

	std::vector<Fan *> allFans = fanContainer->GetAllFans();
	if (allFans.size() > 0)
	{
		for (unsigned int i = 0; i < allFans.size(); i++) 
		{
			Fan *fan = allFans.at(i);

			// we need to set the intensity before deactivate
			if(!activate)
			{
				error = fan->SetIntensityToDevice(0.0f);
			}
			
			fan->ActivateFan(activate);				

			// we need to set the intensity after activate
			if (activate)
			{
				error = fan->SetIntensityToDevice((float)fan->GetOldIntensityValue());
			}
		}
	}
	// force an update
	//if(!activate) 
	//	this->update_amBX();
	devChanged = true;
	ReleaseMutex(globalLock);
#endif
}

/**
Method for activating all vibration devices
\param activate true for activating else false for deactivate
*/
void EffectManager::ActivateAllRumblers(bool activate)
{
#if __USEAMBX__
	WaitForSingleObject(globalLock,INFINITE);	// wait until the current effect changes are made and then disable the appropriate effekt
	std::vector<Rumble *> allRumbles = rumbleContainer->GetAllRumbles();
	if (allRumbles.size() > 0)
	{
		for (unsigned int i = 0; i < allRumbles.size(); i++) 
		{
			Rumble *rumble = allRumbles.at(i);

			// we need to set the intensity before deactivate
			if(!activate)
			{
				error = rumble->SetIntensityToDevice(0.0f);
			}
			
			rumble->ActivateRumble(activate);		
			
			// we need to set the intensity after activate
			if (activate)
			{
				error = rumble->SetIntensityToDevice((float)rumble->GetOldIntensityValue());
			}
		}
	}

	// force an update
	//if(!activate) 
	//	this->update_amBX();
	devChanged = true;
	ReleaseMutex(globalLock);
#endif
}

/**
Method for assigning a color to a device
\param location to assign the color to
\param r red component
\param g green component
\param b blue component
*/
void EffectManager::ColorToDevice(std::string location, int r, int g, int b)
{
#if __USEAMBX__
	int clampedColorR = Utils::Clamp(r, 0, 255);
	int clampedColorG = Utils::Clamp(g, 0, 255);
	int clampedColorB = Utils::Clamp(b, 0, 255);
	
	std::vector<amBX_loc> locations = amBXLocationMap.GetAmBXLocations(location);

	if (locations.size() > 0)
	{
		for (unsigned int i = 0; i < locations.size(); i++)
		{		
			Light *light = lightContainer->GetLight(locations.at(i));
			if(light != NULL) 
				light->SetLightColor(clampedColorR, clampedColorG, clampedColorB);
		}
	}
#endif
}

/**
Method for calculating the percentage of a 8-bit value
(note diffision by 255)
\param value to be transformed
\return percentage (value / 255)
*/
float EffectManager::CalculatePercent(int value)
{
	return value/255.0f;
}

/**
Method for checking if the given intensity is within the defined range
\param value containing the intensity
\param range vector with the range
\return true if in range else false
*/
bool EffectManager::IsInIntensityRange(double value, std::vector<std::string> range) 
{
	if (!range.empty()) 
	{
		if (range.size() == 2) {
			double lower = atof(range.at(0).c_str());
			double upper = atof(range.at(1).c_str());
			if (lower > value || upper < value) 
			{
				printf("Intensity value is outside of the intensity range! Aborting!");
				return false;
			}
		}
		else
			printf("Intensity range is not given correctly!");
	}
	return true;
}

#if __USEAMBX__
/**
Method for handling fading of wind effects
\param time for calculating if fade is needed
\param fan effect to be faded
*/
void EffectManager::FadeWind(double time, Fan *fan)
{

	if ((time-EFFECT_TIME_SPAN) < (fan->GetPTS() + fan->GetFadeTime())) 
	{
		devChanged = true;
	
		fan->SetIntensity(fan->GetIntermediateIntensity() + ((fan->GetEffectDifference() / fan->GetFadeTime()) * (time - fan->GetPTS())));
			
		if (fan->GetIntensity() > -1.0)
			error = fan->SetIntensityToDevice((float)fan->GetIntensity());
		
		if (fan->GetDuration() > 0) 
		{
			fan->SetDuration(fan->GetDuration() - (time - fan->GetPTS()));
			if (fan->GetDuration() <= 0) 
			{
				error = fan->SetIntensityToDevice(0.0);
			}
		}
	}

}

/**
Method for handling fading of vibration effects
\param time for calculating if fade is needed
\param rumble effect to be faded
*/
void EffectManager::FadeVibration(double time, Rumble *rumble)
{
	if ((time-EFFECT_TIME_SPAN) < (rumble->GetPTS() + rumble->GetFadeTime())) 
	{
		devChanged = true;
	
		rumble->SetIntensity(rumble->GetIntermediateIntensity() + ((rumble->GetEffectDifference() / rumble->GetFadeTime()) * (time - rumble->GetPTS())));
			
		if (rumble->GetIntensity() > -1.0)
			error = rumble->SetIntensityToDevice((float)rumble->GetIntensity());
		
		if (rumble->GetDuration() > 0) 
		{
			rumble->SetDuration(rumble->GetDuration() - (time - rumble->GetPTS()));
			if (rumble->GetDuration() <= 0) 
			{
				error = rumble->SetIntensityToDevice(0.0);
			}
		}
	}

}
#endif

/**
Method for checking if any device is enabled
\return true if at least one device is active else false for deactivate
*/
bool EffectManager::AnyDeviceEnabled()
{
	bool oneActivated = false;
#if __USEAMBX__
	amBX_RESULT err;

	// check rumbles
	std::vector<Rumble *> allRumbles = rumbleContainer->GetAllRumbles();
	if (allRumbles.size() > 0)
	{
		for (unsigned int i = 0; i < allRumbles.size() && !oneActivated; i++) 
		{
			Rumble *rumble = allRumbles.at(i);

			amBX_float intensity, speed;
			amBX_u32 length;
			amBX_u32 typeLength = 255;
			amBX_char type[255];
			err = rumble->GetRumble()->getRumble(rumble->GetRumble(), typeLength, type, &length, &speed, &intensity);

			if (err == amBX_OK && intensity > 0.0f)
				oneActivated = true;
		}
	}

	// check fans
	if (!oneActivated)
	{
		std::vector<Fan *> allFans = fanContainer->GetAllFans();
		if (allFans.size() > 0)
		{
			for (unsigned int i = 0; i < allFans.size() && !oneActivated; i++) 
			{
				Fan *fan = allFans.at(i);

				amBX_float intensity;
				err = fan->GetFan()->getIntensity(fan->GetFan(), &intensity);
				if (err == amBX_OK && intensity > 0.0f)
					oneActivated = true;
			}
		}
	}

	// check lights
	if (!oneActivated)
	{
		std::vector<Light *> allLights = lightContainer->GetAllLights();
		if (allLights.size() > 0)
		{
			for (unsigned int i = 0; i < allLights.size() && !oneActivated; i++) 
			{
				Light *light = allLights.at(i);

				amBX_float red, green, blue;
				err = light->GetLight()->getCol(light->GetLight(), &red, &green, &blue);

				if (err == amBX_OK && (red > 0.0f || green > 0.0f || blue > 0.0f))
					oneActivated = true;
			}
		}
	}
#endif

#if __USEVORTEX__
	// check scent
	if (!oneActivated)
	{
		int *scentActive = scentContainer->GetActivationStates();
		for (int i = 0; i < sizeof(scentActive) && !oneActivated; i++)
		{
			if (scentActive[i] == 1)
				oneActivated = true;
		}
	}
#endif

	return oneActivated;
}

/**
Method for activating all devices at once
\param activate true if devices should be activated else false for deactivate
*/
void EffectManager::ActivateAllAvailableDevices(bool activate)
{
#if __USEAMBX__
	ActivateAllRumblers(activate);
	ActivateAllLights(activate);
	ActivateAllFans(activate);

	// force update if deactivated
	if(!activate)
		this->Update_amBX();
#endif
#if __USEVORTEX__
	ActivateAllScents(scentContainer, activate);
#endif
}

/**
Method for first time activation of all devices
needed as else no updates can be send to devices
\param activate true if devices should be activated else false for deactivate
*/
void EffectManager::InitialActivateAllAvailableDevices(bool activate)
{
#if __USEAMBX__
	ActivateAllRumblers(activate);
	ActivateAllLights(activate);
	ActivateAllFans(activate);

	// force update if deactivated
	if(!activate)
		this->Update_amBX();
#endif

// scent not activated here as it will activate all fans
}

/**
Method for detecting all attached devices
*/
void EffectManager::DetectAttachedDevices()
{
#if __USEAMBX__
	// check for lights, fans and rumbler at same location
	if (pIamBX != NULL)
	{
		for (int i = 0; i < amBXLocationMap.GetEnumLocationsLength(); i++)
		{
			IamBX_Light* apIamBXLight = NULL;
			IamBX_Fan* apIamBXFan = NULL;
			IamBX_Rumble* apIamBXRumble = NULL;

			amBX_loc location = (amBX_loc)amBXLocationMap.GetEnumLocations()[i];

			error = pIamBX->createLight(pIamBX, 
								(amBX_loc)location, 
								(amBX_height)amBX_EVERYHEIGHT, 
								&(apIamBXLight));

			if (error == amBX_OK)
			{
				apIamBXLight->setEnabled(apIamBXLight, true);
				apIamBXLight->setCol(apIamBXLight, 0.0f, 0.0f, 0.0f);

				if (!bDisableFateTime)
					error = apIamBXLight->setFadeTime(apIamBXLight, DEFAULT_LIGHTFADE_TIME);

				apIamBXLight->setUpdateProperties(apIamBXLight,LIGHT_MIN_UPDATE_INTERVAL, 0.0f);

				Light *light = new Light(apIamBXLight);
				lightContainer->AddLight((amBX_loc)location, light);
			}

			error = pIamBX->createFan(pIamBX, 
								(amBX_loc)location, 
								(amBX_height)amBX_EVERYHEIGHT, 
								&(apIamBXFan));

			if (error == amBX_OK)
			{
				apIamBXFan->setEnabled(apIamBXFan, true);
				apIamBXFan->setIntensity(apIamBXFan, 0.0f);
				apIamBXFan->setUpdateProperties(apIamBXFan,FAN_MIN_UPDATE_INTERVAL, 0.0f);

				Fan *fan = new Fan(apIamBXFan);
				fanContainer->AddFan((amBX_loc)location, fan);
			}

			error = pIamBX->createRumble(pIamBX, 
								(amBX_loc)location, 
								(amBX_height)amBX_EVERYHEIGHT, 
								&(apIamBXRumble));

			if (error == amBX_OK)
			{
				apIamBXRumble->setEnabled(apIamBXRumble, true);
				apIamBXRumble->setRumble(apIamBXRumble,"amBX_road", 5.0f, 0.0f);
				apIamBXRumble->setUpdateProperties(apIamBXRumble, RUMBLE_MIN_UPDATE_INTERVAL, 0.0f, 0.0f);

				Rumble *rumble = new Rumble(apIamBXRumble);
				rumbleContainer->AddRumble((amBX_loc)location, rumble);
			} 
		}
	}
#endif
}

/**
Method for activating automatic color calculation
\param activate true if devices should be activated else false for deactivate
*/
void EffectManager::ActivateAutoColor(bool activate)
{
	lightAutoColor = activate;
	oldLightAutoColor = activate;
}

/**
Method for stopping all fans
*/
void EffectManager::StopAllFans()
{
#if __USEAMBX__
	// just stop the fans
	std::vector<Fan *> allFans = fanContainer->GetAllFans();
	if (allFans.size() > 0)
	{
		for (unsigned int i = 0; i < allFans.size(); i++) 
		{
			Fan *fan = allFans.at(i);
			fan->SetIntensityToDevice(0.0f);
			// this also kills fading 
			fan->ActivateFade(false);
		}
	}

	devChanged = true;		
#endif
}

/**
Method for stopping all rumblers
*/
void EffectManager::StopAllRumblers()
{
#if __USEAMBX__
	// just stop the rumblers
	std::vector<Rumble *> allRumbles = rumbleContainer->GetAllRumbles();
	if (allRumbles.size() > 0)
	{
		for (unsigned int i = 0; i < allRumbles.size(); i++) 
		{
			Rumble *rumble = allRumbles.at(i);			
			rumble->SetIntensityToDevice(0.0f);
			// this also kills fading 
			rumble->ActivateFade(false);			
		}
	}

	devChanged = true;
#endif
}

#if __USEVORTEX__
/**
Method for activating all scent devices
\param scentCont container to be handled
\param activate true for activating else false for deactivate
*/
void EffectManager::ActivateAllScents(ScentContainer *scentCont, bool activate)
{
	ActivateAllScentDeviceFans(scentCont, activate, true);	
}
#endif

/**
Method for handling scent effects
\param e effect to be handled
*/
void EffectManager::HandleScent(Element e)
{
#if __USEVORTEX__
	if (scentContainer != NULL)
	{
		if (strcmp(e.GetValue(ATT_GEN_ACTIVATE).c_str(), SEMP_TRUE) == 0)
		{
			// check if intensity value is in range
			std::vector<std::string> range;

			if (!strcmp(e.GetValue(ATT_INTENSITYRANGE).c_str(), "") == 0)
			{
				range = Utils::StringSplit(e.GetValue(ATT_INTENSITYRANGE), " ");
				if (!IsInIntensityRange(atof(e.GetValue(ATT_INTENSITYVALUE).c_str()), range))
					return;
			}

			double scentPTS = e.GetPlayedTime();
			double scentFade = atof(e.GetValue(ATT_GEN_FADE).c_str());
			double scentDuration = atof(e.GetValue(ATT_GEN_DURATION).c_str());
			double scentIntensity = atof(e.GetValue(ATT_INTENSITYVALUE).c_str());
			std::string scentType = e.GetValue(ATT_SCENT_SCENT);

			// calculate the percentage of intensity
			if (range.size() == 2)
			{
				double upperBound = atof(range.at(1).c_str());
				scentIntensity = (100.0 / upperBound) * scentIntensity;
			}

			// check if fade is not bigger than duration
			if (scentFade != 0 && scentDuration != 0)
			{
				if (scentFade > scentDuration) 
				{
					printf("Fade cannot be bigger than the duration of effect! Aborting!");
					return;
				}
			}

			if (!scentType.empty())
			{
				Scent *scent = scentContainer->GetScent(scentType);
				if (scent != NULL)
					ActivateScentDeviceFan(scentContainer, scentType, true);
			}
			else // if no scent is specified all scents will be activated
			{
				FT_STATUS status = ActivateAllScentDeviceFans(scentContainer, true);
				if (status != FT_OK)
				{
					printf("An error occured during activation!\n");
				}
			}
		}
		else if (strcmp(e.GetValue(ATT_GEN_ACTIVATE).c_str(), SEMP_FALSE) == 0)
		{
			std::string scentType = e.GetValue(ATT_SCENT_SCENT);

			if (!scentType.empty())
			{
				Scent *scent = scentContainer->GetScent(scentType);
				if (scent != NULL)
					ActivateScentDeviceFan(scentContainer, scentType, false);
			}
			else // if no scent is specified all scents will be deactivated
			{
				FT_STATUS status = ActivateAllScentDeviceFans(scentContainer, false);
				if (status != FT_OK)
				{
					printf("An error occured during deactivation!\n");
				}
			}
		}
	}
#endif
}


// part for handling vortexactive scent device
#if __USEVORTEX__
/**
Method for opening a scent device
\param device to be opened
\return true if successful else false
*/
bool EffectManager::OpenScentDevice(int device)
{
	bool retVal = true;
	FT_HANDLE *deviceHandle = new FT_HANDLE;
	FT_STATUS status = FT_Open(device, deviceHandle);
	
	if (status != FT_OK)
	{
        printf("Please check USB Connection\n");
		retVal = false;
	}
	else
		scentContainer->SetDeviceHandle(deviceHandle);

	return retVal;
}

/**
Method for calculating the Vortex Activ activation code
\param activeFans array with fan states
\return activation code
*/
int EffectManager::CalculateVortexActiveCode(int *activeFans)
{
	if (activeFans != NULL)
		return VORTEXACTIVE_CODEPAGE - VORTEXACTIVE_FAN1*activeFans[0] -
			VORTEXACTIVE_FAN2*activeFans[1] - VORTEXACTIVE_FAN3*activeFans[2] -
			VORTEXACTIVE_FAN4*activeFans[3];
	else
		return 255; // if no fans are found we return the disable command
}

/**
Method for initialising Vortex Activ
\param scentConfiguration path to the configuration file
*/
void EffectManager::InitializeVortexActive(std::string scentConfiguration)
{
	// read config file with available scents
	std::ifstream infile;
	infile.open(scentConfiguration.c_str());
	if (!infile)
	{
		printf("Unable to open file!\n");
		return;
	}

    while(!infile.eof()) // To get you all the lines.
    {
		std::string line;
		std::getline(infile, line); // Saves the line.
	    
		std::vector<std::string> comps = Utils::StringSplit(line, "=");
		// pos 0 = fanid
		// pos 1 = scent urn
		if (comps.size() == 2)
		{
			Scent *scent = new Scent(atoi(comps.at(0).c_str()), comps.at(1));
			scentContainer->AddScent(comps.at(1), scent);
		}
    }
	infile.close();
}

/**
Method for activating all scent device fans
\param scentCont scent container to be handled
\param activate true for activating fans else false for deactivating
\param backup true if current state should be stored else false
\return true if successful else false
*/
bool EffectManager::ActivateAllScentDeviceFans(ScentContainer *scentCont, bool activate, bool backup)
{
	bool retVal = true;
	unsigned char data;
	if (activate)
	{
		// reset activation states
		if (backup)
		{
			scentCont->RestoreActivationStates();
			data = (unsigned char)CalculateVortexActiveCode(scentCont->GetActivationStates());
		}
		else
			data = (unsigned char)0;
	}
	else
	{
		// save old activation states
		if (backup)
			scentCont->BackupActivationStates();

		data = (unsigned char)255; // deactivate all
	}
    	
	if ((retVal = OpenScentDevice(0)))
	{
		DWORD bytesWritten;
		if (scentCont->GetDeviceHandle() != NULL)
		{
			FT_STATUS status = FT_Write(*scentCont->GetDeviceHandle(), &data, sizeof(data), &bytesWritten);
			if (status != FT_OK)
			{
				if (activate)
					printf("An error occured during activation!\n");
				else
					printf("An error occured during deactivation!\n");

				retVal = false;
			}
		}
		else
			retVal = false;
	}

	retVal = CloseScentDevice(scentCont);

	return retVal;
}

/**
Method for closing a scent device
\param scentCont scent container to be handled
\return true if successful else false
*/
bool EffectManager::CloseScentDevice(ScentContainer *scentCont)
{
	bool retVal = true;	
	if (scentCont->GetDeviceHandle() != NULL)
	{
		FT_STATUS status = FT_Close(*scentCont->GetDeviceHandle());
		if (status != FT_OK)
		{
			printf("Unable to Close Port\n");
			retVal = false;
		}
	}

	return retVal;
}

/**
Method for activating a single scent device fan
\param scentCont scent container to be handled
\param scent to be activated
\param activate true for activating else false for deactivating
\return true if successful else false
*/
bool EffectManager::ActivateScentDeviceFan(ScentContainer *scentCont, std::string scent, bool activate)
{
	bool retVal = true;
	Scent *scentToHandle = scentCont->GetScent(scent);

	if (activate)
	{
		scentToHandle->SetIntensity(1.0);
		scentToHandle->SetOldIntensityValue(0.0);
	}
	else
	{
		scentToHandle->SetIntensity(0.0);
		scentToHandle->SetOldIntensityValue(1.0);
	}

	if ((retVal = OpenScentDevice(0)))
	{
		unsigned char data = (unsigned char)CalculateVortexActiveCode(scentCont->GetActivationStates());

		DWORD bytesWritten;
		if (scentCont->GetDeviceHandle() != NULL)
		{
			FT_STATUS status = FT_Write(*scentCont->GetDeviceHandle(), &data, sizeof(data), &bytesWritten);
			if (status != FT_OK)
			{
				printf("An error occured during scent handling!\n");
				retVal = false;
			}
		}
		else
			retVal = false;
	}

	retVal = CloseScentDevice(scentCont);

	return retVal;
}
#endif
