/*
 * This file is part of SEMP.
 *
 * SEMP is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SEMP is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SEMP. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2009, 2010, 2011, 2012 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */
/*
 * Note that this file is based on: readier1.c from LibXML
 */

#include <stdio.h>
#include <algorithm>
#include "Libxml.h"
#include "typedef.h"

#ifdef LIBXML_READER_ENABLED

/**
Method for processing a node
\param reader with the reader object
 */
void Libxml::ProcessNode(xmlTextReaderPtr reader) {
    const xmlChar *name;
    int iType = -1;

    name = xmlTextReaderConstName(reader);
    if (name == NULL || name[0] == '#')
		return;

	std::string name2 = (const char*)name;
	int pos = name2.find(":");
	name = xmlStrsub(name, pos+1, xmlStrlen(name));


    if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_END_ELEMENT)
    {
		if (strcmp((const char*)name, LIBXML_GOE) == 0)
		{
			m_curTime = 0;
			m_anchorElement = false;
		}
    }

    if (xmlTextReaderNodeType(reader) == XML_ELEMENT_NODE)
    {
		// this if-else statements are used for speed-up (reducing string compares)
		if (strcmp((const char*)name, LIBXML_SEM) == 0)
			iType = Libxml::iSEM;
		else if (strcmp((const char*)name, LIBXML_GOE) == 0)
			iType = Libxml::iGOE;
		else if (strcmp((const char*)name, LIBXML_EFFECT) == 0)
			iType = Libxml::iEFFECT;

		while (xmlTextReaderMoveToNextAttribute(reader))
		{
			xmlChar* attr = xmlTextReaderName(reader);

			// needed for removing namespace
			std::string attr2 = (const char*)attr;
			pos = attr2.find(":");
			attr = xmlStrsub(attr, pos+1, xmlStrlen(attr));

			xmlChar* value = xmlTextReaderValue(reader);

			// needed for removing namespace only from type attribute
			if (strcmp((const char*)attr, LIBXML_TYPE) == 0)
			{
				attr2 = (const char*)value;
				pos = attr2.find(":");
				value = xmlStrsub(value, pos+1, xmlStrlen(value));
			}
			
			if (iType == Libxml::iSEM) // SEM
			{
				if (strcmp((const char*)attr, LIBXML_TIMESCALE) == 0)
				{
					const char* val = (const char*)value;
					long int lVal = atol(val);
					m_timeScale = lVal;
				}
			}
			else if (iType == Libxml::iGOE) // GroupOfEffects
			{
				if (strcmp((const char*)attr, LIBXML_ANCHORELEMENT) == 0)
				{
					const char* val = (const char*)value;
					if (strcmp(val, "true") == 0)
						m_anchorElement = true;
					else
						m_anchorElement = false;
				}
				else if (strcmp((const char*)attr, LIBXML_PTS) == 0)
				{
					const char* val = (const char*)value;
					long int lVal = atol(val);

					char buffer[1024];
					sprintf_s(buffer, "%.1f", ( lVal / (double)m_timeScale ));
					m_curTime = atof(buffer);
				}
			}
			else if (iType == Libxml::iEFFECT) // Effect
			{
				if (m_tmpElement == NULL)
				{
					m_tmpElement = new Element();
					m_tmpElement->SetPlayedTime(-1.0);
					// if we are in a GroupOfEffects set the time to the group time
					if (m_anchorElement)
					{
						if (m_curTime != -1.0)
							m_tmpElement->SetTime(m_curTime);
					}
				}

				if (strcmp((const char*)attr, LIBXML_TYPE) == 0)
				{
					if (strcmp((const char*)value, LIBXML_WINDTYPE) == 0)
						m_tmpElement->SetType(TYPE_FAN);
					else if (strcmp((const char*)value, LIBXML_VIBRATIONTYPE) == 0)
						m_tmpElement->SetType(TYPE_VIBRATOR);
					else if (strcmp((const char*)value, LIBXML_LIGHTTYPE) == 0)
						m_tmpElement->SetType(TYPE_LIGHT);
					else if (strcmp((const char*)value, LIBXML_WATERSPRAYERTYPE) == 0)
						m_tmpElement->SetType(TYPE_WATERSPRAYER);
					else if (strcmp((const char*)value, LIBXML_TEMPERATURETYPE) == 0)
						m_tmpElement->SetType(TYPE_AIRCONDITIONER);
					else if (strcmp((const char*)value, LIBXML_FOGTYPE) == 0)
						m_tmpElement->SetType(TYPE_FOG);
					else if (strcmp((const char*)value, LIBXML_SCENTTYPE) == 0)
						m_tmpElement->SetType(TYPE_PERFUMER);
					else
						m_tmpElement->SetType(TYPE_NONE);
				}
				else if (strcmp((const char*)attr, LIBXML_PTS) == 0)
				{
					const char* val = (const char*)value;
					long int lVal = atol(val);
					char buffer[1024];
					sprintf_s(buffer, "%.1f", ( lVal / (double)m_timeScale ));
					m_tmpElement->SetTime(atof(buffer));
				}
				else if (strcmp((const char*)attr, LIBXML_ANCHORELEMENT) == 0)
				{
					// currently ignored
				}
				else if (strcmp((const char*)attr, ATT_GEN_DURATION) == 0)
				{
					const char* val = (const char*)value;
					long int lVal = atol(val);

					char buffer[1024];
					sprintf_s(buffer, "%.1f", (lVal / (double)m_timeScale));
					m_tmpElement->AddValue((const char*)attr, (const char*)buffer);
				}
				else if (strcmp((const char*)attr, ATT_GEN_FADE) == 0)
				{
					const char* val = (const char*)value;
					long int lVal = atol(val);

					char buffer[1024];
					sprintf_s(buffer, "%.1f", (lVal / (double)m_timeScale));
					m_tmpElement->AddValue((const char*)attr, (const char*)buffer);
				}
				else 
				{
					if (m_tmpElement->GetType() == TYPE_LIGHT && strcmp((const char*)attr, ATT_GEN_AUTOEXTRACTION) == 0)
					{
						if (strcmp((const char*)value, VISUAL) == 0 || strcmp((const char*)value, BOTH) == 0)
							m_bAutoColor = true;
					}

					m_tmpElement->AddValue((const char*)attr, (const char*)value);
				}
			}
		}

		if (m_tmpElement != NULL)
		{
			m_Elements->push_back(m_tmpElement);
			m_tmpElement = NULL;
		}
    }
}

/**
Method for parsing and printing information about an XML file
\param filename to the file to be parsed
 */
void Libxml::StreamFile(const char *filename) {
    xmlTextReaderPtr reader;
    int ret;

    reader = xmlReaderForFile(filename, NULL, 0);
    if (reader != NULL) {
        ret = xmlTextReaderRead(reader);
        while (ret == 1) {
            ProcessNode(reader);
            ret = xmlTextReaderRead(reader);
        }
        xmlFreeTextReader(reader);
        if (ret != 0) {
            fprintf(stderr, "%s : failed to parse\n", filename);
        }
    } else {
        fprintf(stderr, "Unable to open %s\n", filename);
    }
}

/**
Constructor
*/
Libxml::Libxml()
{
	m_Elements = new std::vector<Element*>();
	m_curTime = 0;
	m_timeScale = 1;
	m_anchorElement = false;
	m_tmpElement = NULL;
	m_bAutoColor = false;
}

/**
Desctructor
*/
Libxml::~Libxml()
{
	if (m_Elements != NULL)
	{
		delete m_Elements;
		m_Elements = NULL;
	}
}

/**
Method for comparing two elements
\param element1
\param element2
\return true if time of element1 is smaller than time of element2
*/
bool Libxml::ElementCompare(const Element* element1, const Element* element2)
{
	return element1->GetTime() < element2->GetTime();
}

/**
Method for determining if auto color is activated
\return true if auto color is activated else false
*/
bool Libxml::IsAutoColorActivated()
{
	return m_bAutoColor;
}

/**
Method for parsing an XML file
\param filename to the file to parse
\return parsed elements
*/
std::vector<Element*> Libxml::Parse(const char* filename)
{
    LIBXML_TEST_VERSION

    StreamFile(filename);

    xmlCleanupParser();
    xmlMemoryDump();

    std::sort(m_Elements->begin(), m_Elements->end(), ElementCompare);

    /*for (int i = 0; i < m_Elements->size(); i++)
    {
		Element* e = m_Elements->at(i);
		e->print();
    }*/

    return *m_Elements;
}

/**
Method for retrieving the time scale
\return time scale
*/
long Libxml::GetTimeScale()
{
	return m_timeScale;
}

#endif

