/*
 * This file is part of SEMP.
 *
 * SEMP is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SEMP is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SEMP. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2009, 2010, 2011, 2012 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */

#include "ScentContainer.h"

#if __USEVORTEX__
/**
Constructor
*/
ScentContainer::ScentContainer()
{
	deviceHandle = NULL;
}

/**
Destructor
*/
ScentContainer::~ScentContainer()
{
	if (deviceHandle != NULL)
	{
		delete deviceHandle;
		deviceHandle = NULL;
	}
}

/**
Method for adding a scent with a specific type
\param scentType type of the scent
\param scent with the scent to be added
*/
void ScentContainer::AddScent(std::string scentType, Scent* scent)
{
	scentMap.insert(std::make_pair(scentType, scent));	  
}

/**
Method for checking if a scent device is active
\return true if a device is active else false
*/
bool ScentContainer::IsDeviceActive()
{
	bool retVal = false;
	for( std::map<std::string, Scent*>::iterator it = scentMap.begin(); it != scentMap.end() && !retVal; ++it ) {
        Scent *scent = it->second;
		if (scent->GetIntensity() > 0.0)
			retVal = true;
    }

	return retVal;
}

/**
Method for retrieving a scent
\param scentType type of scent to be retrieved
\return scent
*/
Scent* ScentContainer::GetScent(std::string scentType)
{
	std::map<std::string, Scent*>::iterator iter = scentMap.find(scentType);
	if(iter == scentMap.end())
		return NULL;
	else
		return iter->second;
}

/**
Method for retrieving all scents
\return vector with all scents
*/
std::vector<Scent *> ScentContainer::GetAllScents()
{
	std::vector<Scent *> retVal;

	for( std::map<std::string, Scent*>::iterator it = scentMap.begin(); it != scentMap.end(); ++it ) {
        retVal.push_back(it->second);
    }

	return retVal;
}

/**
Method for setting the device handler
\param handle with the device handler
*/
void ScentContainer::SetDeviceHandle(FT_HANDLE *handle)
{
	deviceHandle = handle;
}

/**
Method for retrieving the device handler
\return device handler
*/
FT_HANDLE *ScentContainer::GetDeviceHandle()
{
	return deviceHandle;
}

/**
Method for determining the activation states of the devices
\return array with the activation states
*/
int *ScentContainer::GetActivationStates()
{
	int *activeScents = new int[scentMap.size()];

	for( std::map<std::string, Scent*>::iterator it = scentMap.begin(); it != scentMap.end(); ++it ) {
        Scent *scent = it->second;
		if (scent->GetIntensity() > 0.0)
			activeScents[scent->GetFanID()] = 1; // this scent is activated
		else
			activeScents[scent->GetFanID()] = 0; // this scent is inactive
    }

	return activeScents;
}

/**
Method for storing the activation states and reseting them
*/
void ScentContainer::BackupActivationStates()
{
	for( std::map<std::string, Scent*>::iterator it = scentMap.begin(); it != scentMap.end(); ++it ) {
        Scent *scent = it->second;
		scent->SetOldIntensityValue(scent->GetIntensity());
		scent->SetIntensity(0.0); // clear intensity
    }
}

/**
Method for restoring the activation states and reseting the backup
*/
void ScentContainer::RestoreActivationStates()
{
	for( std::map<std::string, Scent*>::iterator it = scentMap.begin(); it != scentMap.end(); ++it ) {
        Scent *scent = it->second;
		scent->SetIntensity(scent->GetOldIntensityValue());
		scent->SetOldIntensityValue(0.0); // clear old intensity
    }
}
#endif